/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis;

import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Util;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

public class Augenfarbe {

    /**
     * Sortiert die Augenfarben alphabetisch nach Namen.
     */
    public static final Comparator< Augenfarbe > NAME_COMPARATOR =
        new Comparator< Augenfarbe >() {

            public int compare( final Augenfarbe a1, final Augenfarbe a2 ) {
                
                return Util.compare( a1.getName(), a2.getName() );
            }
    };
    
    /** Eine Liste aller Augenfarben. */
    private static final Vector< Augenfarbe > AUGENFARBEN = 
        new Vector< Augenfarbe >();

    /** Eine Map in der unter jeder Augenfarbe ihre Id gespeichert ist. */
    private static final Map< String, Augenfarbe > ID_AUGENFARBE_MAP =
        new HashMap< String, Augenfarbe >();

    public static final Augenfarbe AMETHYST = new Augenfarbe( "Amethyst" );
    public static final Augenfarbe BLAU = new Augenfarbe( "blau" );
    public static final Augenfarbe BRAUN = new Augenfarbe( "braun" );
    public static final Augenfarbe ROT = new Augenfarbe( "rot" );
    public static final Augenfarbe VIOLETT = new Augenfarbe( "violett" );

    static {
        
        final String[] ids = {
                "Amethyst",
                "bernsteinfarben",
                "blau",
                "braun",
                "dunkelbraun",
                "dunkelviolett",
                "grau",
                "graublau",
                "goldgesprenkelt",
                "gruen",
                "hellbraun",
                "rot",
                "saphirblau",
                "schwarz",
                "schwarzbraun",
                "smaragdgruen",
                "violett" };

        for ( final String id : ids ) {

            final Augenfarbe augenfarbe = new Augenfarbe( id );
            
            AUGENFARBEN.add( augenfarbe);

            ID_AUGENFARBE_MAP.put( id, augenfarbe);            
        }
    }

    /** @return eine alphabetisch sortierte Liste mit Augenfarben. */
    public static Vector< Augenfarbe > getAugenfarben() {
        return new Vector< Augenfarbe >( AUGENFARBEN );
    }

    /**
     * @param idOderAugenfarbe
     *            Hier kann die Id einer Augenfarbe uebergeben werden. Es wird
     *            dann die zugehoerige Augenfarbe geliefert. Es kann aber auch
     *            direkt der Name der Augenfarbe gegeben werden. Dann wird
     *            eine neue Augenfarbe mit dem Namen als ID erzeugt.
     *            Wird eine ID statt des direkten Namens uebergeben, so kann
     *            die Augenfarbe lokalisiert werden.
     * @return die Augenfarbe zur ID.
     */
    public static Augenfarbe getAugenfarbe(
            final String idOderAugenfarbe ) {

        if ( ! ID_AUGENFARBE_MAP.containsKey( idOderAugenfarbe ) ) {

            final Augenfarbe augenfarbe = new Augenfarbe( idOderAugenfarbe );
            
            ID_AUGENFARBE_MAP.put( idOderAugenfarbe, augenfarbe );
            
            return augenfarbe;
        
        } else {
        
            return ID_AUGENFARBE_MAP.get( idOderAugenfarbe );
        }
    }
    
    /** Die Id die auch zum lokalisieren verwendet wird. */
    private final String id;
    
    private Augenfarbe( final String dieId ) {

        id = dieId;
    }
    
    @Override
    public boolean equals( Object obj ) {
        
        if ( null == obj ) {
            
            return false;
        }
        
        if ( ! getClass().equals( obj.getClass() ) ) {
            
            return false;
        }
        
        final Augenfarbe augenfarbe = (Augenfarbe) obj;
        
        return getId().equals( augenfarbe.getId() );
    }

    public String getId() {
        
        return id;
    }

    /**
     * @return Die Bezeichnung der Augenfarbe.
     */
    public String getName() {
        
        if ( I18n.hatLokalisierungstext( id ) ) {
            
            return I18n.i18n( id );
        
        } else {
            
            return id;
        }
    }

    @Override
    public String toString() {
        
        return getName();
    }
}
