/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis;

import java.util.ArrayList;
import java.util.List;

import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;

public class Eigenschaftsprobe {

    /** Die erste Probeneigenschaft. */
    private final Eigenschaft eigenschaft1;

    /** Die zweite Probeneigenschaft. */
    private final Eigenschaft eigenschaft2;

    /** Die dritte Probeneigenschaft. */
    private final List< Eigenschaft > eigenschaften;

    /**
	 * @param dieEigenschaft1
	 *            Die erste Probeneigenschaft.
	 *            Wenn es, wie bei der Ritualkenntnis, keine feste Probe gibt,
	 *            so muss hier mindestens die Eigenschaft uebergeben werden,
	 *            nach der die Maximalstufe bestimmt wird.
	 * @param dieEigenschaft2
	 *            Die zweite Probeneigenschaft.
	 *            Es kann {@code null} uebergeben werden, wenn es immer
	 *            verschiedene Proben gibt, wie bei der Ritualkenntnis.
	 * @param dieEigenschaften
	 *            Die dritte Probeneigenschaft oder eine Liste von
	 *            Eigenschaften, falls die Probe nicht immer gleich ist, wie
	 *            zB beim Talent Sinnenschaerfe.
	 *            Die Liste kann leer sein, wenn es fuer das Talent/den Zauber
	 *            keine feste Probe gibt, wie z.B. bei der Ritualkenntnis.
	 */
    public Eigenschaftsprobe(
            final Eigenschaft dieEigenschaft1,
            final Eigenschaft dieEigenschaft2,
            final Eigenschaft ... dieEigenschaften ) {

        eigenschaft1 = dieEigenschaft1;

        eigenschaft2 = dieEigenschaft2;

        eigenschaften = new ArrayList< Eigenschaft >( dieEigenschaften.length );

        for ( final Eigenschaft eigenschaft : dieEigenschaften ) {
        	eigenschaften.add( eigenschaft );
        }
    }

    /**
     * @return die Eigenschaften auf die bei der Probe gewuerfelt wird.
     */
    public final List< Eigenschaft > getEigenschaften() {

    	final List< Eigenschaft > liste =
    		new ArrayList< Eigenschaft >( 2 + eigenschaften.size() );

    	liste.add( eigenschaft1 );

    	if ( null != eigenschaft2 ) {
    	    liste.add( eigenschaft2 );
    	}

    	liste.addAll( eigenschaften );

        return liste;
    }

    /** @return etwas wie '(MU/GE/KK)'. */
    @Override
    public final String toString() {

        if ( null == eigenschaft2 ) {

            return "(*/*/*)";

        } else if ( 1 == eigenschaften.size() ) {

            return String.format( "(%1$s/%2$s/%3$s)",
                    eigenschaft1.getKuerzel(),
                    eigenschaft2.getKuerzel(),
                    eigenschaften.get( 0 ).getKuerzel() );

        } else {

            return String.format( "(%1$s/%2$s/*)",
                    eigenschaft1.getKuerzel(),
                    eigenschaft2.getKuerzel() );
        }
    }
}
