/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis;

import static de.bernhardjung.dsaprogramm.daten.xml.FernwaffeXmlTags.ATTR_ID;
import static de.bernhardjung.dsaprogramm.daten.xml.FernwaffeXmlTags.ATTR_NAME;
import static de.bernhardjung.dsaprogramm.daten.xml.FernwaffeXmlTags.ATTR_REICHWEITEN;
import static de.bernhardjung.dsaprogramm.daten.xml.FernwaffeXmlTags.ATTR_TALENT;
import static de.bernhardjung.dsaprogramm.daten.xml.FernwaffeXmlTags.ATTR_TP;
import static de.bernhardjung.dsaprogramm.daten.xml.FernwaffeXmlTags.ATTR_TP_PLUS;
import static de.bernhardjung.dsaprogramm.daten.xml.FernwaffeXmlTags.TAG_FERNWAFFE;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Talent;

public class Fernkampfwaffe
extends ElementMitNameImpl{
    
    /**
     * Die Reichweiten der Waffe. Sehr nah steht an Position 0, extrem weit an
     * Position 4.
     */
    private final List< Integer > reichweiten;

    /** Das Talent mit dem die Waffe gefuehrt wird. */
    private Kampftechnik talent;

    /** Die Trefferpunkte der Waffe. */
    private Tp tp;

    /**
     * Die Trefferpunkte durch Entfernung. Die TP fuer sehr nah stehen an
     * Position 0, die fuer extrem weit an Position 4.
     */
    private final List< Integer > tpPlus;

    /**
     * Erzeugt keine richtige Fernkampfwaffe, sondern nur ein Dummy der 
     * verwendet werden kann wenn es keine vom Benutzer eingegebene Waffe gibt. 
     */
    public Fernkampfwaffe() {
        this( "WaffeDummy" );
    }

    public Fernkampfwaffe( final String dieId ) {
        super( dieId );

        reichweiten = new ArrayList< Integer >( 5 );
        Collections.fill( reichweiten, 0 );

        talent = Talent.BOGEN;

        tp = new Tp( "W+4" );

        tpPlus = new ArrayList< Integer >( 5 );
        Collections.fill( tpPlus, 0 );
    }

    /**
     * @return die Reichweiten der Waffe. Sehr nah ist an Position 0, extrem
     *         weit an Position 4.
     */
    public List< Integer > getReichweiten() {
        return new ArrayList< Integer >( reichweiten );
    }

    /**
     * @return das Talent mit dem die Waffe gefuehrt wird.
     */
    public Kampftechnik getTalent() {
        return talent;
    }

    /** @return die TP der Waffe. */
    public Tp getTp() {
        return tp;
    }

    /**
     * @return die TP+ der Waffe. Sehr nah ist an Position 0, extrem
     *         weit an Position 4.
     */
    public List< Integer > getTpPlus() {
        return new ArrayList< Integer >( tpPlus );
    }
    
    /**
     * @param technik
     *            Die Kampftechnik mit der diese Waffe gefuehrt werden soll.
     * @return {@code true}, wenn man die Waffe mit dem Talent ohne Abzuege
     *         fuehren kann, sonst {@code false}. Bei anderen Talenten gibt
     *         es einen Abzug von 5 Punkten auf den Fernkampfwert.
     */
    public final boolean hatHaupttalent( final Kampftechnik technik ) {
        return talent.equals( technik );
    }

    /**
     * @param neueReichweiten fuer die Waffe. Sehr nah steht an Position 0,
     *          extrem weit an Position 4.
     */
    public void setReichweiten( final List< Integer > neueReichweiten ) {
        reichweiten.clear();

        reichweiten.addAll( neueReichweiten );
    }

    /**
     * @param neuesTalent mit dem die Waffe gefuehrt wird.
     */
    public void setTalent( final Kampftechnik neuesTalent ) {
        talent = neuesTalent;
    }

    /**
     * @param neueTp der Waffe.
     */
    public void setTp( final Tp neueTp ) {
        tp = neueTp;
    }

    /**
     * @param neueTpPlus fuer die Waffe. Sehr nah steht an Position 0,
     *          extrem weit an Position 4.
     */
    public void setTpPlus( final List< Integer > neueTpPlus ) {
        tpPlus.clear();

        tpPlus.addAll( neueTpPlus );
    }

    @Override
    public String toString() {
        return getName();
    }

    /**
     * Wandelt eine Liste in einen String um, in dem die Listenelemente durch
     * Komma getrennt sind.
     */
    private String toString( final List< Integer > liste ) {

        final StringBuilder builder = new StringBuilder();

        builder.append( liste.get( 0 ) );

        for ( int i = 1, n = liste.size() ; i < n ; ++i ) {

            builder.append( "," );
            builder.append(  liste.get( i ) );
        }

        return builder.toString();
    }

    public IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_FERNWAFFE );

        element.setAttribute( ATTR_ID, getId() );

        if ( null != name ) {

            element.setAttribute( ATTR_NAME, name );
        }

        element.setAttribute( ATTR_REICHWEITEN, toString( reichweiten ) );

        element.setAttribute( ATTR_TALENT, talent.getId() );

        element.setAttribute( ATTR_TP, tp.toString() );

        element.setAttribute( ATTR_TP_PLUS, toString( tpPlus ) );

        return element;
    }
}
