/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis;

import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.i18n.I18n;

public enum Lernmethode {

    KEINE( "Lernmethode.Keine" ),
    SELBSTSTUDIUM ( "Lernmethode.Selbststudium" ),
    GEGENSEITIG ( "Lernmethode.Gegenseitig" ),
    LEHRMEISTER( "Lernmethode.Lehrmeister" ),
    SPEZIELLE_ERFAHRUNG( "Lernmethode.SpezielleErfahrung" ),
    FREI( "Lernmethode.Frei" );

    /** 
     * Der Schluessel mit dem der Name der Lenmethode aus der 
     * Lokalisierungsdatei geladen wird.
     */
    private final String lokalisierungsschluessel;
    
    /**
     * @param dieId der Lernmethode
     */
    private Lernmethode( final String dieId ) {

        lokalisierungsschluessel = dieId + ".Name";
    }

    /**
     * Liefert den Steigerungsfaktor eines Regelobjekts, modifiziert nach dieser
     * Lernmethode.
     *
     * @param steigerbar
     *            Objekt dessen Steigerungsfaktor ermittelt werden soll.
     * @return den Steigerungsfaktor des Regelobjekts nach der Lernmethode.
     */
    public final int bestimmeSteigerungsfaktor(
            final Steigerbar steigerbar )
    {
        return bestimmeSteigerungsfaktor( steigerbar, null );
    }

    /**
     * Liefert den Steigerungsfaktor eines Regelobjekts, modifiziert nach dieser
     * Lernmethode.
     *
     * @param steigerbar
     *            Objekt dessen Steigerungsfaktor ermittelt werden soll.
     * @return den Steigerungsfaktor des Regelobjekts nach der Lernmethode.
     */
    public final int bestimmeSteigerungsfaktor(
            final Steigerbar steigerbar,
            Variante variante ) {

        switch ( this ) {
            case SPEZIELLE_ERFAHRUNG:
            case LEHRMEISTER:
                return ( -1 + steigerbar.getSteigerungsspalte( variante ) );

            case GEGENSEITIG:
            case KEINE:
            case FREI:
                return steigerbar.getSteigerungsspalte( variante );

            case SELBSTSTUDIUM:
                return ( 1 + steigerbar.getSteigerungsspalte( variante ) );

            default:
                throw new RuntimeException( String.format(
                        "Die Lernmethode %1$s ist noch nicht in die Methode " +
                        "eingebunden.", this.getName() ) );
        }
    }
    
    /**
     * @return Den Lokalisierungsschluessel um die Uebersetzung des Namens der
     *          Lernmethode zu laden.
     */
    public final String getL10nschluesselName() {
        
        return lokalisierungsschluessel;
    }

    /** @return den Namen der Lernmethode, zB "Spezielle Erfahrung". */
    public final String getName() {
        return I18n.i18n( lokalisierungsschluessel );
    }

    /**
     * @return den Namen der Lernmethode.
     */
    @Override
    public String toString() {
        return getName();
    }
}
