/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis;

import java.util.ArrayList;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.util.Util;

public class NahkampfwaffeWert implements Cloneable {

    /**
     * Erzeugt eine Deep Copy der uebergebenene Werte.
     */
    public static ArrayList< NahkampfwaffeWert > deepCopy(
            final Collection< NahkampfwaffeWert > originale ) {
        
        final ArrayList< NahkampfwaffeWert > kopien =
            new ArrayList< NahkampfwaffeWert >( originale.size() );
        
        for ( final NahkampfwaffeWert wert : originale ) {
            
            kopien.add( wert.clone() );
        }
        
        return kopien;
    }
    
    public static final String DK_H = I18n.i18n( "NahkampfwaffeWert.DkH" );

    public static final String DK_N = I18n.i18n( "NahkampfwaffeWert.DkN" );

    public static final String DK_S = I18n.i18n( "NahkampfwaffeWert.DkS" );

    /** Der Bruchfaktor der Waffe. */
    private int bf;

    /** Distanzklasse Handegemenge */
    private boolean dkH;

    /** Distanzklasse Nahkampf */
    private boolean dkN;

    /** Distanzklasse Stangenwaffe */
    private boolean dkS;

    /** Der Initiativemodifikator der Waffe. */
    private int ini;

    /** Der Typ von Waffe, um den es sich hier handelt. */
    private Nahkampfwaffe nahkampfwaffe;

    /** Der Name der Waffe. */
    private String name;

    /** Der Schadensschritt fuer die TP/KK. */
    private int schadensschritt;

    /** Der Schwellenwert fuer TP/KK. */
    private int schwellenwert;

    /** Das Talent mit denen die Waffe gefuehrt wird. */
    private Kampftechnik talent;

    /** Die Formel fuer die Trefferpunkte. */
    private Tp tp;

    /** Der Waffenmodifikator auf Attacke. */
    private int wmAt;

    /** Der Waffenmodifikator auf Parade */
    private int wmPa;

    public NahkampfwaffeWert() {
        this( new Nahkampfwaffe() );
    }

    public NahkampfwaffeWert( final Nahkampfwaffe waffe ) {

        setWaffe( waffe );
    }
    
    /**
     * @return Eine Deep Copy des Wertes. 
     */
    protected NahkampfwaffeWert clone() {
        
        NahkampfwaffeWert clone = null;
        
        try {
            
            clone = (NahkampfwaffeWert) super.clone();
            
        } catch ( CloneNotSupportedException e ) {
            
            DialogFehlermeldung.zeigeDialog( e );
        }
        
        return clone;
    }
    
    @Override
    public boolean equals( Object obj ) {
        
        if ( null == obj 
                || ! obj.getClass().equals( getClass() ) ) {
            
            return false;
        }
        
        final NahkampfwaffeWert wert = (NahkampfwaffeWert) obj;
        
        return Util.equals( 
                bf, wert.bf,
                dkH, wert.dkH,
                dkN, wert.dkN,
                dkS, wert.dkS,
                ini, wert.ini,
                nahkampfwaffe, wert.nahkampfwaffe,
                name, wert.name,
                schadensschritt, wert.schadensschritt,
                schwellenwert, wert.schwellenwert,
                talent, wert.talent,
                tp, wert.tp,
                wmAt, wert.wmAt,
                wmPa, wert.wmPa );
    }

    /**
     * Liefert den Attackewert des Helden mit dieser Waffe.
     *
     * @param held
     *            dessen Attackewert gesucht ist.
     * @return den Attackewert.
     */
    public final int getAttacke( final Held held ) {

        int at = held.getStufe( Basiswert.ATB );

        at += wmAt;

        if ( held.hat( talent ) ) {

            at += held.getWert( talent ).getAttacke();

        }

        if ( ! held.hat( talent )
          || ! nahkampfwaffe.hatHaupttalent( talent ) ) {

            at -= 2;
        }

        if ( hatSpezialisierung( held ) ) {

            ++at;
        }

        int be = held.getBehinderung();

        be -= talent.getEBe();

        be /= 2;

        if ( 0 < be ) {

            at -= be;
        }

        return at;
    }

    /** @return den Bruchfaktor der Waffe. */
    public final int getBf() {
        return bf;
    }

    /** @return einen String zum Anzeigen der Distanzklasse(n) der Waffe. */
    public final String getDkString() {

        final StringBuilder string = new StringBuilder();

        if ( dkH ) {

            string.append( DK_H );
        }

        if ( dkN ) {

            string.append( DK_N );
        }

        if ( dkS ) {

            string.append( DK_S );
        }

        return string.toString();
    }

    /** @return Den Initiativemodifikator der Waffe. */
    public final int getIni() {
        return ini;
    }

    /** @return den Namen der Waffen. */
    public final String getName() {
        return name;
    }

    /**
     * Liefert den Paradewert eines Helden mit dieser Waffe.
     *
     * @param held
     *            dessen Paradewert gesucht ist.
     * @return den Paradewert des Helden mit dieser Waffe.
     */
    public final int getParade( final Held held ) {

        int pa = held.getStufe( Basiswert.PAB );

        pa += wmPa;

        if ( held.hat( talent ) ) {

            pa += held.getWert( talent ).getParade( held );

        }

        if ( ! held.hat( talent )
          || ! nahkampfwaffe.hatHaupttalent( talent ) ) {

            pa -= 3;
        }

        if ( hatSpezialisierung( held ) ) {

            ++pa;
        }

        int be = held.getBehinderung();

        be -= talent.getEBe();

        be = ( be + 1 ) / 2;

        if ( 0 < be ) {

            pa -= be;
        }

        return pa;
    }

    /** @return den Schadensschritt fuer die TP/KK. */
    public final int getSchadensschritt() {
        return schadensschritt;
    }

    /** @return den Schwellenwert fuer die TP/KK. */
    public final int getSchwellenwert() {
        return schwellenwert;
    }

    /** @return Das Talent mit denen die Waffe gefuehrt wird. */
    public final Kampftechnik getTalent() {
        return talent;
    }

    /** @return Die Formel fuer die TP. */
    public final Tp getTp() {
        return tp;
    }

    /**
     * @param held der die Waffe fuehrt.
     * @return die Formel fuer die TP, die der Held mit der Waffe verursacht.
     */
    public final String getTp( final Held held ) {

        final int kk = held.getStufe( Eigenschaft.KK );

        final int tpPlus = ( kk - schwellenwert ) / schadensschritt;

        return tp.getTpFormel( tpPlus );
    }

    /**
     * @return die Waffenart um die es sich bei dieser Waffe handelt.
     */
    public final Nahkampfwaffe getWaffe() {
        return nahkampfwaffe;
    }

    /** @return den Waffenmodifikator auf Attacke. */
    public final int getWmAt() {
        return wmAt;
    }

    /** @return den Waffenmodifikator auf Parade. */
    public final int getWmPa() {
        return wmPa;
    }

    /**
     * Prueft ob der Held eine Spezialisierung fuer diese Waffe besitzt.
     *
     * @param held
     *            der ueberprueft werden soll.
     * @return {@code true}, wenn der Held eine Spezialisierung besitzt, sonst
     *         {@code false}.
     */
    private boolean hatSpezialisierung( final Held held ) {

        final Sonderfertigkeit spezialisierung =
            Sonderfertigkeit.getWaffenspezialisierung( talent );

        final SonderfertigkeitWert wert = new SonderfertigkeitWert( 
                spezialisierung,
                nahkampfwaffe.getLokalisierungsschluesselName() );

        return held.hat( wert );
    }

    /**
     * @return {@code true}, wenn die Waffe in der Distanzklasse Handgemenge
     *         gefuehrt werden kann, sonst {@code false}.
     */
    public final boolean istDkH() {
        return dkH;
    }

    /**
     * @return {@code true}, wenn die Waffe in der Distanzklasse Nahkampf
     *         gefuehrt werden kann, sonst {@code false}.
     */
    public final boolean istDkN() {
        return dkN;
    }

    /**
     * @return {@code true}, wenn die Waffe in der Distanzklasse Stangenwaffe
     *         gefuehrt werden kann, sonst {@code false}.
     */
    public final boolean istDkS() {
        return dkS;
    }

    /** @param neuerBf Der neue Bruchfaktor der Waffe. */
    public final void setBf( final int neuerBf ) {
        bf = neuerBf;
    }

    /**
     * @param handgemenge {@code true}, wenn die Waffe in der Distanzklasse
     *            Handgemenge gefuehrt werden kann, sonst {@code false}.
     */
    public final void setDkH( final boolean handgemenge ) {
        dkH = handgemenge;
    }

    /**
     * @param nahkampf {@code true}, wenn die Waffe in der Distanzklasse
     *            Nahkampf gefuehrt werden kann, sonst {@code false}.
     */
    public final void setDkN( final boolean nahkampf ) {
        dkN = nahkampf;
    }

    /**
     * @param stangenwaffe {@code true}, wenn die Waffe in der Distanzklasse
     *            Handgemenge gefuehrt werden kann, sonst {@code false}.
     */
    public final void setDkS( final boolean stangenwaffe ) {
        dkS = stangenwaffe;
    }

    /** @param neueIni Der neue Initiativemodifikator der Waffe. */
    public final void setIni( final int neueIni ) {
        ini = neueIni;
    }

    /**
     * @param neuerName fuer die Waffe.
     */
    public final void setName( final String neuerName ) {
        name = neuerName;
    }

    /**
     * @param neuerSchadensschritt fuer die TP/KK.
     */
    public final void setSchadensschritt( final int neuerSchadensschritt ) {
        schadensschritt = neuerSchadensschritt;
    }

    /** @param neuerSchwellenwert fuer die TP/KK. */
    public final void setSchwellenwert( final int neuerSchwellenwert ) {
        schwellenwert = neuerSchwellenwert;
    }

    /**
     * @param neuesTalent Das Talent mit denen die Waffe gefuehrt wird.
     */
    public final void setTalent( final Kampftechnik neuesTalent ) {

        talent = neuesTalent;
    }

    /** @param neueTp fuer die Waffe. */
    public final void setTp( final Tp neueTp ) {
        tp = neueTp;
    }

    /**
     * Bestimmt den Typ von Waffe um den es sich hier handelt. Die Eigenschaften
     * der Waffe, wie TP und BF werden uebernommen.
     *
     * @param waffe
     *            Der Typ der Waffe um den es sich hier handelt.
     */
    public final void setWaffe( final Nahkampfwaffe waffe ) {

        nahkampfwaffe = waffe;

        setBf( waffe.getBf() );

        setDkH( waffe.istDkH() );

        setDkN( waffe.istDkN() );

        setDkS( waffe.istDkS() );

        setIni( waffe.getIni() );

        setName( waffe.getName() );

        setSchadensschritt( waffe.getSchadensschritt() );

        setSchwellenwert( waffe.getSchwellenwert() );

        setTalent( waffe.getTalent() );

        setTp( waffe.getTp() );

        setWmAt( waffe.getWmAt() );

        setWmPa( waffe.getWmPa() );
    }

    /** @param modifikator auf die Attacke. */
    public final void setWmAt( final int modifikator ) {
        wmAt = modifikator;
    }

    /** @param modifikator auf die Parade. */
    public final void setWmPa( final int modifikator ) {
        wmPa = modifikator;
    }

    @Override
    public final String toString() {
        return name;
    }
}
