/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis.heldio;

import java.util.ArrayList;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.*;
import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.basis.Augenfarbe;
import de.bernhardjung.dsaprogramm.basis.FernkampfwaffeWert;
import de.bernhardjung.dsaprogramm.basis.GeldBoerse;
import de.bernhardjung.dsaprogramm.basis.Haarfarbe;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Kulturkunde;
import de.bernhardjung.dsaprogramm.basis.NahkampfwaffeWert;
import de.bernhardjung.dsaprogramm.basis.RuestungWert;
import de.bernhardjung.dsaprogramm.basis.SchildTyp;
import de.bernhardjung.dsaprogramm.basis.SchildWert;
import de.bernhardjung.dsaprogramm.basis.Tp;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.basiswerte.BasiswertWert;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.bausteine.util.Geschlecht;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.eigenschaften.EigenschaftWert;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Sprache;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.TalentWert;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;
import de.bernhardjung.dsaprogramm.zauber.Repraesentation;
import de.bernhardjung.dsaprogramm.zauber.Zauber;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;
import static de.bernhardjung.dsaprogramm.basis.heldio.HeldXmlTags.*;

public class FactoryHeldXmlIn {

    private final Datenspeicher speicher = Datenspeicher.getInstanz();

    /**
     * Ermittelt die Augenfarbe des Helden aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Augenfarbe des Helden.
     */
    private Augenfarbe ermitteleAugenfarbe(
            final IXMLElement wurzelElement ) {

        String augenfarbe =
            wurzelElement.getFirstChildNamed( TAG_AUGENFARBE ).getContent();

        return Augenfarbe.getAugenfarbe( augenfarbe );
    }

    /**
     * Ermittelt die Fernkampfwaffen des Helden aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Fernkampfwaffen, die der Held mit sich fuehrt.
     */
    private List< FernkampfwaffeWert > ermitteleFernkampfwaffen(
            final IXMLElement wurzelElement ) {

        final List< FernkampfwaffeWert > waffen =
            new ArrayList< FernkampfwaffeWert >();

        final IXMLElement waffenElement =
            wurzelElement.getFirstChildNamed( TAG_FERNKAMPFWAFFE_WERTE );

        if ( null == waffenElement ) {
            // nichts zu tun.
            return waffen;
        }

        for ( final IXMLElement waffeElement :
            waffenElement.getChildren() ) {

            final String waffenId =
                waffeElement.getAttribute( ATTR_FERNKAMPFWAFFE );

            final FernkampfwaffeWert waffe =
                new FernkampfwaffeWert( speicher.getFernkampfwaffe( waffenId ));

            waffe.setName( waffeElement.getAttribute( ATTR_NAME ) );

            final String reichweiten =
                waffeElement.getAttribute( ATTR_REICHWEITEN );

            waffe.setReichweiten( toIntegerListe( reichweiten ) );

            if ( waffeElement.hasAttribute( ATTR_TALENT  ) ) {

                final String talentId =
                    waffeElement.getAttribute( ATTR_TALENT );

                waffe.setTalent( (Kampftechnik) Talent.getTalent( talentId ) );
            }

            waffe.setTp( new Tp( waffeElement.getAttribute( ATTR_TP ) ) );

            final String tpPlus =
                waffeElement.getAttribute( ATTR_TP_PLUS );

            waffe.setTpPlus( toIntegerListe( tpPlus ) );

            waffen.add( waffe );
        }

        return waffen;
    }

    /**
     * Ermittelt das Geschlecht des Helden aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return das Geschlecht des Helden/der Heldin.
     */
    private Geschlecht ermitteleGeschlecht( final IXMLElement wurzelElement ){

        final String name =
            wurzelElement.getFirstChildNamed( TAG_GESCHLECHT ).getContent();

        return Geschlecht.valueOf( name );
    }

    /**
     * Ermittelt die Haarfarbe des Helden aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Haarfarbe des Helden.
     */
    private Haarfarbe ermitteleHaarfarbe( final IXMLElement wurzelElement ) {

        String haarfarbe =
            wurzelElement.getFirstChildNamed( TAG_HAARFARBE ).getContent();

        return Haarfarbe.getHaarfarbe( haarfarbe );
    }

    /**
     * Ermittelt die Hauszauber aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Hauszauber des Held. Wenn der Held keine HAuszauber hat ist
     *         die Liste leer.
     */
    private List< Zauber > ermitteleHauszauber(
            final IXMLElement wurzelElement ) {

        final List< Zauber > hauszauber = new ArrayList< Zauber >();

        final IXMLElement hauszauberElement =
            wurzelElement.getFirstChildNamed( TAG_HAUSZAUBER );

        if ( null != hauszauberElement ) {

            for ( final IXMLElement talentElement :
                hauszauberElement.getChildren() ) {

                final String id = talentElement.getContent();

                hauszauber.add( Zauber.getZauber( id ) );
            }
        }

        return hauszauber;
    }
    
    /**
     * Ermittelt das Kapital des Helden aus dem XML-Dokument
     * @param wurzelElement des XML-Dokuments
     * @return gesVermoegen des Helden.
     */
    private int ermitteleKapital( final IXMLElement wurzelElement ) {
        
        final int gesVermoegen;
        String gesVermoegenAlsString;
        
        try {

         gesVermoegenAlsString = 
                wurzelElement.getFirstChildNamed( TAG_KAPITAL ).getContent();
        } catch (NullPointerException npe) {
            gesVermoegenAlsString = "0";
        }
        
        gesVermoegen = Integer.parseInt( gesVermoegenAlsString );
        
        return gesVermoegen;
    }
    
    /**
     * Ermittelt die Kultur des Helden aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Kultur des Helden.
     */
    private Kultur ermitteleKultur( final IXMLElement wurzelElement ) {

        final String id =
            wurzelElement.getFirstChildNamed( TAG_KULTUR ).getContent();

        return Datenspeicher.getInstanz().getKultur( id );
    }

    /**
     * Ermittelt die Leittalente aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Leittalente des Held. Wenn der Held keine Leittalente hat ist
     *         die Liste leer.
     */
    private List< Talent > ermitteleLeittalente(
            final IXMLElement wurzelElement ) {

        final List< Talent > leittalente = new ArrayList< Talent >();

        final IXMLElement leittalenteElement =
            wurzelElement.getFirstChildNamed( TAG_LEITTALENTE );

        if ( null != leittalenteElement ) {

            for ( final IXMLElement talentElement :
                leittalenteElement.getChildren() ) {

                final String id = talentElement.getContent();

                leittalente.add( Talent.getTalent( id ) );
            }
        }

        return leittalente;
    }

    /**
     * Ermittelt die Leitzauber aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Leitzauber des Held. Wenn der Held keine Leitzauber hat ist
     *         die Liste leer.
     */
    private List< Zauber > ermitteleLeitzauber(
            final IXMLElement wurzelElement ) {

        final List< Zauber > leitzauber = new ArrayList< Zauber >();

        final IXMLElement leitzauberElement =
            wurzelElement.getFirstChildNamed( TAG_LEITZAUBER );

        if ( null != leitzauberElement ) {

            for ( final IXMLElement talentElement :
                leitzauberElement.getChildren() ) {

                final String id = talentElement.getContent();

                leitzauber.add( Zauber.getZauber( id ) );
            }
        }

        return leitzauber;
    }

    /**
     * Ermittelt die Muttersprache des Helden aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Muttersprache des Helden.
     */
    private Sprache ermitteleMuttersprache( final IXMLElement wurzelElement ) {

        final String id = getTagInhalt( TAG_MUTTERSPRACHE, wurzelElement );

        return (Sprache) Talent.getTalent( id );
    }

    /**
     * Ermittelt die Nahkampfwaffen des Helden aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Nahkampfwaffen des Helden.
     */
    private List< NahkampfwaffeWert > ermitteleNahkampfwaffen(
            final IXMLElement wurzelElement ) {

        final List< NahkampfwaffeWert > waffen =
            new ArrayList< NahkampfwaffeWert >();

        final IXMLElement waffenElement =
            wurzelElement.getFirstChildNamed( TAG_NAHKAMPFWAFFE_WERTE );

        if ( null == waffenElement ) {
            // nichts zu tun.
            return waffen;
        }

        for ( final IXMLElement waffeElement : waffenElement.getChildren() ) {

            final String waffenId =
                waffeElement.getAttribute( ATTR_NAHKAMPFWAFFE );

            final NahkampfwaffeWert waffe =
                new NahkampfwaffeWert( speicher.getNahkampfwaffe( waffenId ) );

            waffe.setBf( getAttributInt( waffeElement, ATTR_BF ) );

            waffe.setDkH( getAttributBoolean( waffeElement, ATTR_DK_H ) );

            waffe.setDkN( getAttributBoolean( waffeElement, ATTR_DK_N ) );

            waffe.setDkS( getAttributBoolean( waffeElement, ATTR_DK_S ) );

            waffe.setIni( getAttributInt( waffeElement, ATTR_INI ) );

            waffe.setName( waffeElement.getAttribute( ATTR_NAME ) );

            waffe.setSchadensschritt(
                    getAttributInt( waffeElement, ATTR_SCHADENSSCHRITT ) );

            waffe.setSchwellenwert(
                    getAttributInt( waffeElement, ATTR_SCHWELLENWERT ) );

            final String talentId = waffeElement.getAttribute( ATTR_TALENT );

            waffe.setTalent( (Kampftechnik) Talent.getTalent( talentId ) );

            waffe.setTp( new Tp( waffeElement.getAttribute( ATTR_TP ) ) );

            waffe.setWmAt( getAttributInt( waffeElement, ATTR_WM_AT ) );

            waffe.setWmPa( getAttributInt( waffeElement, ATTR_WM_PA ) );

            waffen.add( waffe );
        }

        return waffen;
    }

    /**
     * Ermittelt die Profession des Helden aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Profession des Helden.
     */
    private Profession ermitteleProfession( final IXMLElement wurzelElement ){

        final String id =
            wurzelElement.getFirstChildNamed( TAG_PROFESSION ).getContent();

        return Datenspeicher.getInstanz().getProfession( id );
    }

    /**
     * Ermittelt die Rasse des Helden aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Rasse des Helden.
     */
    private Rasse ermitteleRasse( final IXMLElement wurzelElement ) {

        final String id =
            wurzelElement.getFirstChildNamed( TAG_RASSE ).getContent();

        return Datenspeicher.getInstanz().getRasse( id );
    }

    /**
     * Ermittelt die Ruestungen, die der Held traegt, aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Ruestungen des Helden.
     */
    private List< RuestungWert > ermitteleRuestungen(
            final IXMLElement wurzelElement ) {

        final List< RuestungWert > ruestungen =
            new ArrayList< RuestungWert >();

        final IXMLElement ruestungenElement =
            wurzelElement.getFirstChildNamed( TAG_RUESTUNG_WERTE );

        if ( null == ruestungenElement ) {
            // nichts zu tun.
            return ruestungen;
        }

        for ( final IXMLElement ruestungElement :
            ruestungenElement.getChildren() ) {

            final String ruestungsId =
                ruestungElement.getAttribute( ATTR_RUESTUNG );

            final RuestungWert ruestung =
                new RuestungWert( speicher.getRuestung( ruestungsId ) );

            ruestung.setBe( getAttributInt( ruestungElement, ATTR_BE ) );

            ruestung.setName( ruestungElement.getAttribute( ATTR_NAME ) );

            ruestung.setRs( getAttributInt( ruestungElement, ATTR_RS ) );

            ruestungen.add( ruestung );
        }

        return ruestungen;
    }

    /**
     * Ermittelt die Schilde, die der Held mit sich traegt, aus dem
     * XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Schilde des Helden.
     */
    private List< SchildWert > ermitteleSchilde(
            final IXMLElement wurzelElement ) {

        final List< SchildWert > schilde =
            new ArrayList< SchildWert >();

        final IXMLElement schildeElement =
            wurzelElement.getFirstChildNamed( TAG_SCHILD_WERTE );

        if ( null == schildeElement ) {
            // nichts zu tun.
            return schilde;
        }

        for ( final IXMLElement schildElement :
            schildeElement.getChildren() ) {

            final String schildId =
                schildElement.getAttribute( ATTR_SCHILD );

            final SchildWert schild =
                new SchildWert( speicher.getSchild( schildId ) );

            schild.setBf( getAttributInt( schildElement, ATTR_BF ) );

            schild.setIni( getAttributInt( schildElement, ATTR_INI ) );

            schild.setName( schildElement.getAttribute( ATTR_NAME ) );

            final String typId = schildElement.getAttribute( ATTR_TYP );

            schild.setTyp( SchildTyp.getTyp( typId ) );

            schild.setWmAt( getAttributInt( schildElement, ATTR_WM_AT ) );

            schild.setWmPa( getAttributInt( schildElement, ATTR_WM_PA ) );

            schilde.add( schild );
        }

        return schilde;
    }

    /**
     * Ermittelt eine Sonderfertigkeit aus einem XML-IXMLElement.
     *
     * @param sfElement
     *            das IXMLElement, das die Sonderfertigkeit repraesentiert.
     * @return eine Sonderfertigkeit.
     */
    private Sonderfertigkeit ermitteleSonderfertigkeit(
            final IXMLElement sfElement ) {

        final String id =
            sfElement.getAttribute( ATTR_SONDERFERTIGKEIT );

        final Sonderfertigkeit sf;

        if ( id.equals( Sonderfertigkeit.ID_SCHARFSCHUETZE ) ) {

            final String talentId = sfElement.getAttribute( ATTR_TALENT );

            final Talent talent = Talent.getTalent( talentId );

            sf = Sonderfertigkeit.getScharfschuetze( (Kampftechnik) talent );

        } else if ( id.equals( Sonderfertigkeit.ID_TALENTSPEZIALISIERUNG ) ) {

            final String talentId = sfElement.getAttribute( ATTR_TALENT );

            final Talent talent = Talent.getTalent( talentId );

            sf = Sonderfertigkeit.getTalentspezialisierung( talent );

        } else if ( id.equals ( Sonderfertigkeit.ID_WAFFENSPEZIALISIERUNG ) ) {

            final String talentId = sfElement.getAttribute( ATTR_TALENT );

            final Talent talent = Talent.getTalent( talentId );

            sf = Sonderfertigkeit.getWaffenspezialisierung(
                    (Kampftechnik) talent );
        } else {

            sf = Sonderfertigkeit.getSonderfertigkeit( id );
        }

        return sf;
    }

    private List< SonderfertigkeitWert > ermitteleSonderfertigkeiten(
            final IXMLElement wurzelElement ) {

        final List< SonderfertigkeitWert > werte =
            new ArrayList< SonderfertigkeitWert >();

        final IXMLElement werteElement =
            wurzelElement.getFirstChildNamed( TAG_SONDERFERTIGKEIT_WERTE );

        if ( null != werteElement ) {

            for ( final IXMLElement wertElement :
                werteElement.getChildren() ) {

                werte.add( erzeugeSonderfertigkeitWert( wertElement ) );
            }
        }

        return werte;
    }
    /**
     * Ermittelt die Talentwerte aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Talentwerte des Helden.
     */
    private List< TalentWert > ermitteleTalentwerte(
            final IXMLElement wurzelElement ) {

        final List< TalentWert > werte = new ArrayList< TalentWert >();

        final IXMLElement werteElement =
            wurzelElement.getFirstChildNamed( TAG_TALENT_WERTE );

        for ( final IXMLElement wertElement : werteElement.getChildren() ) {

            final String id = wertElement.getAttribute( ATTR_TALENT );

            final Talent talent = Talent.getTalent( id );

            final int stufe =
                getAttributInt( wertElement, ATTR_UNMODIFIZIERTE_STUFE );

            final TalentWert wert = new TalentWert( talent, stufe );

            final boolean spezielleErfahrung =
                getAttributBoolean( wertElement, ATTR_SPEZIELLE_ERFAHRUNG );

            wert.setSpezielleErfahrung( spezielleErfahrung );

            final String aktiviertString =
                wertElement.getAttribute( ATTR_AKTIVIERT );

            if ( null != aktiviertString ) {

                wert.setAktiviert( Boolean.valueOf( aktiviertString ) );
            }

            final String attackeString =
                wertElement.getAttribute( ATTR_ATTACKE );

            if ( null != attackeString ) {

                wert.setAttacke( Integer.valueOf( attackeString ) );
            }

            werte.add( wert );
        }

        return werte;
    }

    /**
     * Ermittelt die verbilligten Sonderfertigkeiten aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die verbilligten Sonderfertigkeiten. Wenn der Held keine
     *         verbilligten Sonderfertigkeiten hat, so ist die Liste leer.
     */
    private List< Sonderfertigkeit > ermitteleVerbilligteSonderfertigkeiten(
            final IXMLElement wurzelElement ) {

        final List< Sonderfertigkeit > verbilligte =
            new ArrayList< Sonderfertigkeit >();

        final IXMLElement sfenElement =
            wurzelElement.getFirstChildNamed(
                    TAG_VERBILLIGTE_SONDERFERTIGKEITEN );

        if ( null != sfenElement ) {

            for ( final IXMLElement sfElement : sfenElement.getChildren() ) {

                verbilligte.add(
                    ermitteleSonderfertigkeit( sfElement ) );

            }
        }

        return verbilligte;
    }

    /**
     * Ermittelt die Sonderfertigkeiten, von denen eine Variante verbilligt ist,
     * aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return Die verbilligten Varianten des Helden. Wenn der Held keine
     *         Varianten von Sonderfertigkeiten verbilligt bekommt, so ist die
     *         Liste leer.
     */
    private List< SonderfertigkeitWert > ermitteleVerbilligteVarianten(
            final IXMLElement wurzelElement ) {

        final List< SonderfertigkeitWert > varianten =
            new ArrayList< SonderfertigkeitWert >();

        final IXMLElement variantenElement =
            wurzelElement.getFirstChildNamed( TAG_VERBILLIGTE_VARIANTEN );

        if ( null != variantenElement ) {

            for ( final IXMLElement wertElement :
                variantenElement.getChildren() ) {

                varianten.add(
                        erzeugeSonderfertigkeitWert( wertElement ) );
            }
        }

        return varianten;
    }

    /**
     * Ermittelt die Vor-/Nachteile aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return eine Liste mit den Vor-/Nachteilen des Helden. Wenn der Held
     *         keine Vor-/Nachteile hat, so ist die Liste leer.
     */
    private List< VorNachteilWert > ermitteleVorNachteile(
            final IXMLElement wurzelElement ) {

        final List< VorNachteilWert > werte =
            new ArrayList< VorNachteilWert >();

        final IXMLElement werteElement =
            wurzelElement.getFirstChildNamed( TAG_VOR_NACHTEIL_WERTE );

        if ( null != werteElement ) {

            for ( final IXMLElement wertElement : werteElement.getChildren() ){

                werte.add( erzeugeVorNachteilWert( wertElement ) );
            }
        }

        return werte;
    }

    /**
     * Ermittelt die Zauber des Helden aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return Eine Liste der Zauber des Helden. Die Liste ist leer, wenn der
     *         Held keine Zauber beherrscht.
     */
    private List< ZauberWert > ermitteleZauber(
            final IXMLElement wurzelElement ) {

        final List< ZauberWert > werte = new ArrayList< ZauberWert >();

        final IXMLElement zauberElement =
            wurzelElement.getFirstChildNamed( TAG_ZAUBER_WERTE );

        if ( null != zauberElement ) {

            for ( final IXMLElement element : zauberElement.getChildren() ) {

                final String id = element.getAttribute( ATTR_ZAUBER );

                final Zauber zauber = Zauber.getZauber( id );

                final int stufe = getAttributInt( element, ATTR_STUFE );

                 Object variante = null;
                if ( element.hasAttribute( ATTR_VARIANTE ) ) {

                    variante = element.getAttribute( ATTR_VARIANTE );

                } else {

                    IXMLElement varianteElt =
                            element.getFirstChildNamed( ATTR_VARIANTE );
                    if ( varianteElt != null ) {
                        String name = varianteElt.getAttribute( ATTR_NAME );
                        Variante v = new Variante( name );

                        v.setProbe( erzeugeProbe(
                                varianteElt.getFirstChildNamed( TAG_PROBE ) ) );

                        v.setRepraesentationen( getAttributRepraesentationen(
                                varianteElt,
                                ATTR_REPRAESENTATION ) );

                        v.setSteigerungsspalte( getAttributInt(
                                varianteElt,
                                ATTR_STEIGERUNGSSPALTE ) );

                        variante = v;
                    }
                }

                final ZauberWert wert;

                if ( null == variante ) {

                    wert = new ZauberWert( zauber, stufe );

                } else {

                    wert = new ZauberWert( zauber, variante, stufe );
                }

                final boolean spezielleErfahrung =
                    getAttributBoolean( element, ATTR_SPEZIELLE_ERFAHRUNG );

                wert.setSpezielleErfahrung( spezielleErfahrung );

                werte.add( wert );
            }

        }

        return werte;
    }

    /**
     * Ermittelt die Zweitsprache des Helden aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die Zweitsprache des Helden oder {@code null}, wenn der Held
     *         keine Zweitsprache beherrscht.
     */
    private Sprache ermitteleZweitsprache( final IXMLElement wurzelElement ) {

        final String id = getTagInhalt( TAG_ZWEITSPRACHE, wurzelElement );

        if ( null != id ) {

            return (Sprache) Talent.getTalent( id );

        } else {

            return null;

        }

    }

    /**
     * Wandelt ein XML-Dokument in einen Helden um.
     *
     * @param wurzelElement
     *            Des XML-Dokuments.
     * @return den erzeugten Helden.
     */
    public final Held erzeugen( final IXMLElement wurzelElement ) {

        final Held held = new Held();

        // wirdErschaffen auf false setzen muss vor dem Einfuegen von Werten
        // geschehen, da die Methoden sich dann anders verhalten.
        // Beim Einfuegen von Vor-/Nachteilen arbeiten die Anpasser anders, bzw.
        // nicht mehr.
        held.setWirdErschaffen( false );

        held.setName( getTagInhalt( TAG_NAME, wurzelElement ) );

        // Die Rasse muss vor solchen Sachen wie Haarfarbe gesetzt werden, da
        // diese Dinge beim setzen der Rasse, entsprechend der Rasse,
        // aktualisiert werden.
        held.setRasse( ermitteleRasse( wurzelElement ) );

        held.setKultur( ermitteleKultur( wurzelElement ) );

        held.setProfession( ermitteleProfession( wurzelElement ) );

        held.setGeschlecht( ermitteleGeschlecht( wurzelElement ) );

        held.setGeburtstag( getTagInhalt( TAG_GEBURTSTAG, wurzelElement ) );

        held.setAlter( getTagInhaltInt( TAG_ALTER, wurzelElement ) );

        held.setGroesse( getTagInhaltInt( TAG_GROESSE, wurzelElement ) );

        held.setGewicht( getTagInhaltInt( TAG_GEWICHT, wurzelElement ) );

        held.setHaarfarbe( ermitteleHaarfarbe( wurzelElement ) );

        held.setAugenfarbe( ermitteleAugenfarbe( wurzelElement ) );

        held.setAussehen( getTagInhalt( TAG_AUSSEHEN, wurzelElement ) );

        held.setStand( getTagInhalt( TAG_STAND, wurzelElement ) );

        held.setTitel( getTagInhalt( TAG_TITEL, wurzelElement ) );

        held.setHintergrund( getTagInhalt( TAG_HINTERGRUND, wurzelElement ) );

        held.setApGesamt( getTagInhaltInt( TAG_GESAMT_AP, wurzelElement ) );

        held.setApEingesetzt( getTagInhaltInt( TAG_EINGESETZTE_AP,
                                               wurzelElement ) );

        setEigenschaften( held, wurzelElement );

        setBasiswerte( held, wurzelElement );

        held.setMuttersprache( ermitteleMuttersprache( wurzelElement ) );

        final Sprache zweitsprache = ermitteleZweitsprache( wurzelElement );

        if ( null != zweitsprache ) {

            held.setZweitsprache( zweitsprache );
        }

        for ( final TalentWert wert : ermitteleTalentwerte( wurzelElement ) ) {

            held.einfuegenVorgegeben( wert );
        }

        held.einfuegenLeittalente( ermitteleLeittalente( wurzelElement ) );

        for ( final ZauberWert wert : ermitteleZauber( wurzelElement ) ) {

            held.einfuegenVorgegeben( wert );
        }

        held.einfuegenHauszauber( ermitteleHauszauber( wurzelElement ) );

        held.einfuegenLeitzauber( ermitteleLeitzauber( wurzelElement ) );

        for ( final SonderfertigkeitWert wert :
            ermitteleSonderfertigkeiten( wurzelElement ) ) {

            held.einfuegenVorgegeben( wert );
        }

        for ( final Sonderfertigkeit sonderfertigkeit :
            ermitteleVerbilligteSonderfertigkeiten( wurzelElement ) ) {

            held.einfuegenVerbilligt( sonderfertigkeit );
        }

        for ( final SonderfertigkeitWert wert :
            ermitteleVerbilligteVarianten( wurzelElement ) ) {

            held.einfuegenVerbilligt( wert );
        }

        for ( final VorNachteilWert wert :
            ermitteleVorNachteile( wurzelElement ) ) {

            held.einfuegenVorgegeben( wert );
        }

        for ( final NahkampfwaffeWert waffe :
            ermitteleNahkampfwaffen( wurzelElement ) ) {

            held.einfuegen( waffe );
        }

        for ( final FernkampfwaffeWert waffe :
            ermitteleFernkampfwaffen( wurzelElement ) ) {

            held.einfuegen( waffe );
        }

        for ( final RuestungWert ruestung :
            ermitteleRuestungen( wurzelElement ) ) {

            held.einfuegen( ruestung );
        }

        for ( final SchildWert schild : ermitteleSchilde( wurzelElement ) ) {

            held.einfuegen( schild );
        }
        
        held.setGeldBoerse( new GeldBoerse( ermitteleKapital( wurzelElement ) ) );
        
        return held;
    }

    /**
     * Die Probe eines Talents oder eines Zaubers.
     */
    private Eigenschaftsprobe erzeugeProbe(
            final IXMLElement probeElt ) {

        if ( probeElt == null )
            return null;

        Eigenschaft e1 = getAttributeEigenschaft( probeElt, ATTR_EIGENSCHAFT1 );
        Eigenschaft e2 = getAttributeEigenschaft( probeElt, ATTR_EIGENSCHAFT2 );
        Eigenschaft e3 = getAttributeEigenschaft( probeElt, ATTR_EIGENSCHAFT3 );

        return new Eigenschaftsprobe( e1, e2, e3 );
    }

    /**
     * Erzeugt eine Sonderfertigkeit aus einem IXMLElement des XML-Dokuments.
     *
     * @param wertElement
     *            Das IXMLElement, das den Wert repraesentiert.
     * @return der aus dem IXMLElement ermittelte Wert.
     */
    private SonderfertigkeitWert erzeugeSonderfertigkeitWert(
            final IXMLElement wertElement ) {

        final Sonderfertigkeit sonderfertigkeit =
            ermitteleSonderfertigkeit( wertElement );

        final SonderfertigkeitWert wert;

        if ( sonderfertigkeit.hatVarianten() ) {

            final Object variante;

            final String varianteString;
            if ( wertElement.hasAttribute( ATTR_VARIANTE ) ) {

                varianteString = wertElement.getAttribute( ATTR_VARIANTE );

            } else {

                varianteString = wertElement.getFirstChildNamed( ATTR_VARIANTE ).getAttribute( ATTR_NAME );
            }
            if ( istTalentspezialiserungKulturkundeFehler(
                    sonderfertigkeit,
                    varianteString ) )
                variante = "Talent.Spezialisierung." + varianteString;
            else
                variante = getVariante( varianteString );

            wert = new SonderfertigkeitWert( sonderfertigkeit, variante );

        } else {

            wert = new SonderfertigkeitWert( sonderfertigkeit );
        }

        return wert;
    }

    /**
     * Erzeugt einen Vor-/Nachteil aus einem IXMLElement des XML-Dokuments.
     *
     * @param wertElement
     *            Das IXMLElement, das den Wert repraesentiert.
     * @return der aus dem IXMLElement ermittelte Wert.
     */
    private VorNachteilWert erzeugeVorNachteilWert(
            final IXMLElement wertElement ) {

        final String id = wertElement.getAttribute( ATTR_VOR_NACHTEIL );

        final VorNachteil vorNachteil = VorNachteil.getVorNachteil( id );

        final VorNachteilWert wert;

        if ( vorNachteil.hatVarianten() ) {

            final Variante variante;
            if ( wertElement.hasAttribute( ATTR_VARIANTE ) ) {

                final String varianteString =
                        wertElement.getAttribute( ATTR_VARIANTE );

                 variante = new Variante( varianteString );

            } else {

                IXMLElement varianteElt =
                        wertElement.getFirstChildNamed( ATTR_VARIANTE );
                String name = varianteElt.getAttribute( ATTR_NAME );
                variante = new Variante( name );
                variante.setGpKosten( getAttributInt( wertElement, ATTR_GPKOSTEN ) );
                variante.setMindeststufe( getAttributInt( wertElement, ATTR_MINDESTSTUFE ) );
                variante.setProStufe( getAttributInt( wertElement, ATTR_PRO_STUFE ) );
                variante.setSchrittweite( getAttributInt( wertElement, ATTR_SCHRITTWEITE ) );
                variante.setHatStufen( getAttributBoolean( wertElement, ATTR_HAT_STUFEN ) );
                variante.setHoechststufe( getAttributInt( wertElement, ATTR_HOECHSTSTUFE ) );
            }

            wert = new VorNachteilWert( vorNachteil, variante );

        } else {

            wert = new VorNachteilWert( vorNachteil );
        }

        final String stufeString =
            wertElement.getAttribute( ATTR_STUFE );

        if ( null != stufeString ) {

            wert.setStufe( Integer.valueOf( stufeString ) );
        }

        final String spezielleErfahrung =
            wertElement.getAttribute( ATTR_SPEZIELLE_ERFAHRUNG );

        if ( null != spezielleErfahrung ) {

            wert.setSpezielleErfahrung( Boolean.valueOf( spezielleErfahrung ) );
        }

        return wert;
    }

    /**
     * Liest eine Attribut-Wert aus, der einen Boolean-Wert enthaelt.
     *
     * @param element
     *            dessen Attribut ausgelesen werden soll.
     * @param attribut
     *            das ausgelesen werden soll.
     * @return den Wert des Attributs.
     */
    private boolean getAttributBoolean(
            final IXMLElement element,
            final String attribut ) {

        final String wert = element.getAttribute( attribut, String.valueOf( false ) );

        return Boolean.valueOf( wert );
    }

    /**
     * Liefert die Eigenschaft, deren ID im uebergebenene Attribut
     * gespeichert ist.
     */
    private Eigenschaft getAttributeEigenschaft(
            final IXMLElement element,
            final String attribut ) {

        final String id = element.getAttribute( attribut );

        return Eigenschaft.getEigenschaft( id );
    }

    /**
     * Liest eine Attribut-Wert aus, der einen Int-Wert enthaelt.
     *
     * @param element
     *            dessen Attribut ausgelesen werden soll.
     * @param attribut
     *            das ausgelesen werden soll.
     * @return den Wert des Attributs.
     */
    private int getAttributInt(
            final IXMLElement element,
            final String attribut ) {

        final String wert = element.getAttribute( attribut, "0" );

        return Integer.valueOf( wert );
    }

    /**
     * Liest EINE Repraesentation aus einem Attribute aus, liefert es aber
     * als Liste zurueck.
     */
    private List< Repraesentation > getAttributRepraesentationen(
            final IXMLElement element,
            final String attribut ) {

        final String id = element.getAttribute(  attribut );

        final Repraesentation rep = Repraesentation.getRepraesentation( id );

        if ( null == rep )
            return null;

        List< Repraesentation > reps = new ArrayList< Repraesentation >();
        reps.add( rep );

        return reps;
    }

    /**
     * Liefert den Textinhalt eines Elements des XML-Datei.
     *
     * @param tagName
     *            Der Name des Tags dessen Inhalt gesucht ist.
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return Den Inhalt des ersten Vorkommens des Elements {@code tag} im
     *         XML-Dokument. {@code null}, wenn das Tag nicht vor kommt.
     */
    private String getTagInhalt(
            final String tagName,
            final IXMLElement wurzelElement ) {

        final IXMLElement tag = wurzelElement.getFirstChildNamed( tagName );

        if ( null != tag ) {

            String content = tag.getContent();

            if ( null == content ) {

                content = "";
            }

            return content;

        } else {
            return null;
        }
    }

    /**
     * Liefert den Inhalt eines Elements des XML-Datei, der aus einem
     * Integer-Wert besteht.
     *
     * @param tag
     *            dessen Inhalt gesucht ist.
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return Den Inhalt des ersten Vorkommens des Elements {@code tag} im
     *         XML-Dokument.
     */
    private int getTagInhaltInt(
            final String tag,
            final IXMLElement wurzelElement ) {

        return Integer.valueOf( getTagInhalt( tag, wurzelElement ) );
    }

    private Object getVariante(
            final String varianteString ) {

        if ( varianteString == null )
            return null;

        if ( varianteString.startsWith( "Kulturkunde." ) ) {

            return Kulturkunde.getKulturkunde( varianteString );

        } else if ( varianteString.startsWith( "Repraesentation." ) ) {

            return Repraesentation.getRepraesentation( varianteString );

        } else {

            return varianteString;
        }
    }

    /**
     * In den Versionen bis 0.8.5 hatten die Talentspezialisierungen von
     * Etikette, Menschenkenntnis und Sagen/Legenden IDs der Form
     * "Kulturkunde.XYZ". Die Spezialisierung ist an sich eine Variante
     * vom Typ String. Nach dem Speichern eines Helden mit Sf Talentspezialisierung
     * in einem der Talente wurd aber nicht der String wiederhergestellt,
     * sondern ein Objekt vom Typ Kulturkunde => Exception.
     *
     * Um das zu beheben hab ich erst mal die IDs veraendert.
     *
     * Damit muss bein Import aber die ID veraendert werden und darauf
     * wird hier geprueft.
     */
    private boolean istTalentspezialiserungKulturkundeFehler(
            final Sonderfertigkeit sonderfertigkeit,
            final String varianteString ) {
        return sonderfertigkeit.istTalentspezialisierung()
                && varianteString.startsWith( "Kulturkunde." );
    }

    /**
     * Stellt die Basiswerte des Helden ein.
     *
     * @param held
     *            dessen Werte ermittelt werden.
     * @param wurzelElement
     *            des XML-Dokuments aus dem die Daten gelesen werden.
     */
    private void setBasiswerte(
            final Held held,
            final IXMLElement wurzelElement ) {

        final IXMLElement werteElement =
            wurzelElement.getFirstChildNamed( TAG_BASISWERT_WERTE );

        for ( final IXMLElement wertElement : werteElement.getChildren() ) {

            final String id = wertElement.getAttribute( ATTR_BASISWERT );

            final Basiswert basiswert = Basiswert.getBasiswert( id );

            final BasiswertWert wert = held.getWert( basiswert );

            wert.setBonus( getAttributInt( wertElement, ATTR_BONUS ) );

            wert.setZukaufVorgegeben( getAttributInt(wertElement, ATTR_ZUKAUF));

            wert.setSpezielleErfahrung( getAttributBoolean(
                    wertElement, ATTR_SPEZIELLE_ERFAHRUNG ) );
        }
    }

    /**
     * Stellt die Eigenschaftswerte des Helden ein.
     *
     * @param held
     *            dessen Werte ermittelt werden.
     * @param wurzelElement
     *            des XML-Dokuments aus dem die Daten gelesen werden.
     */
    private void setEigenschaften(
            final Held held,
            final IXMLElement wurzelElement ) {

        final IXMLElement werteElement =
            wurzelElement.getFirstChildNamed( TAG_EIGENSCHAFT_WERTE );

        for ( final IXMLElement wertElement : werteElement.getChildren() ) {

            final String id = wertElement.getAttribute( ATTR_EIGENSCHAFT );

            final Eigenschaft eigenschaft = Eigenschaft.getEigenschaft( id );

            final EigenschaftWert wert = held.getWert( eigenschaft );

            wert.setBasisstufe( getAttributInt( wertElement, ATTR_BASISSTUFE ));

            wert.setZukaufVorgegeben( getAttributInt(wertElement, ATTR_ZUKAUF));

            wert.setSpezielleErfahrung( getAttributBoolean(
                    wertElement, ATTR_SPEZIELLE_ERFAHRUNG ) );
        }
    }

    /**
     * Wandelt einen String, der ein kommaseparierte Liste von Zahlen enthaelt,
     * in eine Liste von Integer Werten um.
     *
     * @param string String der Form "1,2,3".
     * @return den String als Liste.
     */
    private List< Integer > toIntegerListe( final String string ) {

        final String[] array = string.split( "," );

        final List< Integer > liste = new ArrayList< Integer >( array.length );

        for ( final String wert : array ) {

            liste.add( Integer.valueOf( wert ) );
        }

        return liste;
    }
}
