/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bernhardjung.dsaprogramm.basis.varianten;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Eigenschaftsprobe;
import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.basis.ElementMitNameImpl;
import de.bernhardjung.dsaprogramm.basis.SKT;
import de.bernhardjung.dsaprogramm.gui.steigern.PanelEigenschaftenSteigern;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.zauber.Repraesentation;

/**
 * Die Variante hat zwei Hauptverwendungen.
 *
 * 1. Die Variante wird z.B. fuer Sonderfertigkeiten verwendet,
 * die in unterschiedlichen Varianten vorkommen. Es gibt z.B.
 * die Sonderfertigkeit Gelaendekunde und Varianten fuer jedes Gelaende.
 * Erst die Kombination der Sf + der Variaten (dem Gelaende) ergibt
 * eine Sonderfertigkeit, die zu einem Charakter hinzugefuegt werden
 * kann.
 *
 * 2. Fuer vom Benutzer hinzugefuegte Regeln, z.B. einen Vorteil,
 * den es eigentlich nicht in den Regeln gibt, speichert die Variante
 * die Werte, wie z.B. Name und Kosten. Die Variante wird dann
 * mit einem Vorteil kombiniert, der diese Informationen aus der
 * Variante bezieht.
 */
public class Variante {

    /**
     * Sortiert die Varianten alphabetische nach Name.
     */
    public static final Comparator< Variante > NAME_COMPARATOR =
        new Comparator< Variante >() {

            public int compare(
                    final Variante v1,
                    final Variante v2 ) {
                
                return Util.compare( v1.getName(), v2.getName() );
            }
        
    };

    /**
     * Fuer einen vom Benutzer hinzugefuegten Wert, sind hier die AP-Kosten
     * gespeichert
     */
    private int apKosten;

    /** 
     * Hier kann ein Regelelement, z.B. ein Talent oder eine Ruestung 
     * gespeichert werden.
     */
    private final ElementMitName element;

    /**
     * Fuer einen vom Benutzer hinzugefuegten Wert, sind hier die GP-Kosten
     * gespeichert
     */
    private int gpKosten;

    /**
     * Fuer einen vom Benutzer hinzugefuegten Wert, ist hier vermerkt,
     * ob der Wert Stufen hat.
     */
    private boolean hatStufen;

    /**
     * Fuer einen vom Benutzer hinzugefuegten Wert mit Stufen,
     * ist hier vermerkt was die maximal erlaubte Stufe ist.
     */
    private int hoechststufe;

    /**
     * Fuer einen vom Benutzer hinzugefuegten Wert mit Stufen,
     * ist hier vermerkt was die niedrigste erlaubte Stufe ist.
     */
    int mindeststufe;

    /**
     * Fuer ein vom Benutzer hinzugefuegtes Talent/Zauber
     * ist dies die hier die Probe auf die gewuerfelt wird.
     */
    private Eigenschaftsprobe probe;

    /**
     * Fuer einen vom Benutzer hinzugefuegten Wert mit Stufen,
     * ist hier vermerkt pro wieviel Stufen die AP-/GP-Kosten
     * zu zahlen sind. Bei 3 GP pro 2 Punkte, werden die 2 hier vermerkt.
     */
    private int proStufe;

    /**
     * Fuer vom Benutzer hinzugefuegte Zauber.
     */
    private List< Repraesentation > repraesentationen;

    /** 
     * Wenn die Variante ein lokalisierbarer Text ist dies der 
     * Lokalisierungsschluessel ueber den der Name der Variante aus der
     * Lokalisierungsdatei geladen wird. Wenn die Variante aus nur aus einem 
     * nicht lokalisierten Text besteht wird hier der Text gespeichert.  
     */
    private final String schluesselOderName;

    private int schrittweite;

    /** Fuer benutzerdefinierte Talente und Zauber. */
    private int steigerungsspalte;

    public Variante() {

        apKosten = 0;

        element = null;

        gpKosten = 0;

        hatStufen = false;

        hoechststufe = 0;

        mindeststufe = 0;

        proStufe = 1;

        schluesselOderName = "";

        steigerungsspalte = SKT.A;
    }

    public Variante( final ElementMitName dasElement ) {
        
        element = dasElement;
        
        schluesselOderName = "";
    }
    
    /**
     * Hier kann entweder ein Lokalisierungsschluessel oder ein Name uebergeben
     * werden. Es geht auch die ID eines Elements mit Name.
     */
    public Variante( final String id ) {
        
        if ( ElementMitNameImpl.istId( id ) ) {
            
            element = ElementMitNameImpl.getElement( id );
            
            schluesselOderName = "";
        
        } else {
            
            element = null;
            
            if ( I18n.hatLokalisierungstext( id + ".Name" ) ) {
                
                schluesselOderName = id + ".Name";
            
            } else {
                
                schluesselOderName = id;
            }
        }
    }
    
    @Override
    public boolean equals( Object obj ) {
        
        if ( null == obj ) {
            
            return false;
        }
        
        if ( ! getClass().equals( obj.getClass() ) ) {
            
            return false;
        }
        
        final Variante variante = (Variante) obj;

        if ( getGpKosten() != variante.getGpKosten()
                || getHoechststufe() != variante.getHoechststufe()
                || getMindeststufe() != variante.getMindeststufe()
                || getProStufe() != variante.getProStufe()
                || getSchrittweite() != variante.getSchrittweite()
                || hatStufen() != variante.hatStufen() )
            return false;

        if ( hatElementMitName() && variante.hatElementMitName() ) {
            
            return element.equals( variante.element );
        
        } else if ( hatSchluesselOderName() && variante.hatSchluesselOderName() ) {
            
            return schluesselOderName.equals( variante.schluesselOderName );
        
        } else if ( isNull() && variante.isNull() ) {
            
            return true;
        
        } else {
            
            return false;
        }
    }

    public final int getApKosten() {
        return apKosten;
    }

    /**
     * @return Das Element (Talent, Vorteil...) aus dem die Variante beseteht.
     *         Wenn die Variante nur aus einem Text besteht wird {@code null}
     *         zurueckt gegeben.  
     */
    public final ElementMitName getElement() {
        
        return element;
    }

    public final int getGpKosten() {
        return gpKosten;
    }

    public final int getHoechststufe() {

        return hoechststufe;
    }

    /**
     * @return Eine ID, um das Element zu identifizieren. Es ist entweder
     *          der SchluesselOderName oder die ID des ElementMitName.
     */
    public final String getId() {
        
        if ( hatSchluesselOderName() ) {
            
            return schluesselOderName;
        
        } else {
            
            return element.getId();
        }
    }

    public final int getMindeststufe() {

        return mindeststufe;
    }
    
    /**
     * @return Die Bezeichnung der Talentspezialisierung.
     */
    public final String getName() {
        
        if ( hatElementMitName() ) {
            
            return element.getName();
            
        } else if ( I18n.hatLokalisierungstext( schluesselOderName ) ) {
            
            return I18n.i18n( schluesselOderName );
        
        } else {
            
            return schluesselOderName;
        }
    }

    /**
     * Bei einem vom Benutzer hinzugefuegten Talent/Zauber die Probe, auf
     * die gewuerfelt wird.
     */
    public final Eigenschaftsprobe getProbe() {

        return probe;
    }

    /**
     * Die Stufenpunkte pro die z.B. die Kosten eines Vorteils anfallen.
     * Wenn der Vorteil 3 GP pro 2 Punkte kostet, dann liefert diese Methode
     * die 2.
     */
    public final int getProStufe() {
        return proStufe;
    }

    /**
     * Die Reprasentationen eines vom Benutzer hinzugefuegten Zaubers.
     */
    public final List< Repraesentation > getRepraesentationen() {

        if ( repraesentationen == null )
            return null;
        else
            return new ArrayList< Repraesentation >( repraesentationen );
    }

    /**
     * @return Der Schluessel der zur Lokalisierung des Namens der Variante
     *         verwendet wird. Es kann aber auch einfach direkt der Name der
     *         Variante sein, wenn es keinen Lokalisierungsschluessel gibt.
     */
    public final String getSchluesselOderName() {
        
        return schluesselOderName;
    }

    public final int getSchrittweite() {
        return schrittweite;
    }

    /**
     * @return Fuer benutzerdefinierte Talente und Zauber.
     */
    public final int getSteigerungsspalte() {

        return steigerungsspalte;
    }

    @Override
    public int hashCode() {
        
        if ( hatElementMitName() ) {
            
            return 17 + element.hashCode();
        
        } else {
            
            return 17 + schluesselOderName.hashCode();
        }
    }
    
    /**
     * @return {@code true}, wenn die Variante aus einem Regelelement wie einem
     *          Talent besteht.
     */
    public final boolean hatElementMitName() {
        
        return null != element;
    }
    
    /**
     * @return {@code true}, wenn ein Text als Lokalisierungsschluessel oder 
     *          Name gesetzt ist, {@code false} sonst.
     */
    public final boolean hatSchluesselOderName() {
        
        return ! schluesselOderName.equals( "" );
    }

    public final boolean hatStufen() {
        return hatStufen;
    }
    
    /**
     * @return {@code true}, wenn es sich bei der Variante um die leere Variante
     *         handelt, d.h. eine Variante ohne Element mit leerem Text.
     */
    public boolean isNull() {
        
        if ( hatElementMitName() ) {
            
            return false;
        
        } else {
            
            return schluesselOderName.equals( "" );
        }
    }

    /**
     * Legt fuer einen vom Benutzer hinzugefuegten Wert
     * die AP-Kosten fest.
     * @param kosten in Abenteuerpunkten.
     */
    public void setApKosten( int kosten ) {

        this.apKosten = kosten;
    }

    /**
     * Legt fuer einen vom Benutzer hinzugefuegten Wert
     * die GP-Kosten fest.
     *
     * @param kosten in Generierungspunkten.
     */
    public void setGpKosten( int kosten ) {

        this.gpKosten = kosten;
    }

    /**
     * Legt fuer einen vom Benutzer hinzugefuegten Wert
     * fest, ob er Stufen hat, wie z.B. bei einer
     * Schlechten Eigenschaft.
     *
     * @param stufen {@code true} genau dann wenn er Stufen hat.
     */
    public void setHatStufen( boolean stufen ) {

        this.hatStufen = stufen;
    }

    /**
     * Legt fuer einen vom Benutzer hinzugefuegten Wert
     * fest, was die Mindeststufe ist.
     *
     * @param minimum das der Held haben muss.
     */
    public void setMindeststufe( final int minimum ) {

        this.mindeststufe = minimum;
    }

    /**
     * Legt fuer einen vom Benutzer hinzugefuegten Wert
     * mit Stufen fest, welche Stufe der Held maximal haben
     * darf.
     *
     * @param maximum das erlaubt ist.
     */
    public void setHoechststufe( final int maximum ) {

        this.hoechststufe = maximum;
    }

    /**
     * Fuer vom Benutzer hinzugefuegte Talente oder Zauber.
     *
     * @param probe auf die ein Talent/Zauber gewuerfelt wird.
     */
    public void setProbe( final Eigenschaftsprobe probe ) {

        this.probe = probe;
    }

    /**
     * Legt fest, dass bei einem vom Benutzer hinzugefuegten
     * Wert mit Stufen, die Kosten nur alle x Stufen zu
     * entrichten sind.
     *
     * @param pro wieviel Punkte die vollen AP-/GP-Kosten
     *            zu entrichten sind.
     */
    public void setProStufe( int pro ) {

        this.proStufe = pro;
    }

    /**
     * Die Repraesentationen eines vom Benutzer hinzugefuegten Zaubers.
     */
    public void setRepraesentationen(
            final List< Repraesentation > repraesentationen ) {

        this.repraesentationen = repraesentationen;
    }

    /**
     * Die Schrittweite in welcher eine Stufe,
     * eines vom Benutzer hinzugefuegten Wertes mit Stufen,
     * angehoben werden kann.
     *
     * @param punkte um die man die Stufe anheben/senken kann.
     */
    public void setSchrittweite( int punkte ) {

        this.schrittweite = punkte;
    }

    /**
     * Fuer benutzerdefinierte Talente und Zauber.
     *
     * @param steigerungsspalte
     *          Eine der Spalten aus der Steigerungskostentabelle.
     */
    public void setSteigerungsspalte(
            int steigerungsspalte ) {

        this.steigerungsspalte = steigerungsspalte;
    }

    /**
     * @return Liefert den Namen der Spezialisierung.
     */
    @Override
    public String toString() {
        
        return getName();
    }
}
