/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.bonus;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_LEITTALENT;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_TYP;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_BONUS;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_FEST;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.bausteine.bonus.talentlisten.TalentListe;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.TalentWert;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

public class TalentbonusFest implements Talentbonus {

    /**
     * Bestimmt ob es sich bei dem Talentbonus um ein Leittalent handelt.
     * Leittalente sind spezielle Talente elfischer Charaktere.
     * Siehe Nachteil 'Elfische Weltsicht', Regelwerk Seite 94.
     */
    private boolean leittalent;

    /** Die Talente auf die die Stufenboni zu verteilen sind. */
    private final TalentListe talente;

    /** Die Stufenboni die es auf die Talente zu verteilen gilt. */
    private final List< Integer > stufenBoni;

    /**
     * Gibt an ob es sich um einen normalen TalentBonusFest handelt oder
     * um den speziellen Bonus der Profession Entdecker.
     */
    private TalentbonusTyp typ;

    /**
     * Erzeugt einen neuen Bonus bei dem es sich nicht um einen Bonus mit
     * einem Leittalent handelt.
     * Der Bonus ist vom Typ {@link TalentbonusTyp#FEST}.
     *
     * @param dieTalente auf welche die Stufenboni verteilt werden sollen.
     * @param dieStufenBoni welche es auf die Talente zu verteilen gilt.
     */
    public TalentbonusFest(
            final TalentListe dieTalente,
            final List< Integer > dieStufenBoni ) {

        talente = dieTalente;

        stufenBoni = new ArrayList< Integer >( dieStufenBoni );
        Collections.sort( stufenBoni );
        Collections.reverse( stufenBoni );

        leittalent = false;

        typ = TalentbonusTyp.FEST;
    }

    public int compareTo( final Talentbonus o ) {

        return Talent.NAME_COMPARATOR.compare( getTalent(), o.getTalent() );
    }

    /**
     * @return eine Beschreibung des Talentbonus wie 'Singen, Tanzen oder
     *      Zechen +4, ein weiteres +2'.
     */
    private String getBeschreibung() {

        final StringBuilder beschreibung = new StringBuilder();

        beschreibung.append( talente.getBeschreibung() );

        beschreibung.append( String.format( " %1$+d", stufenBoni.get( 0 ) ) );

        final Talentgruppe talentgruppe = talente.getTalentgruppe();

        final String weitere;

        if ( talentgruppe.equals( Talentgruppe.SPRACHEN )
                || talentgruppe.equals( Talentgruppe.SCHRIFTEN ) ) {

            weitere = I18n.i18n( "TalentBonusFest.eineWeitere" );

        } else {

            weitere = I18n.i18n( "TalentBonusFest.einWeiteres" );

        }

        for ( int i = 1, n = stufenBoni.size() ; i < n ; ++i ) {

            beschreibung.append( weitere );

            beschreibung.append( String.format( " %1$+d", stufenBoni.get( i ) ) );
        }

        if ( leittalent ) {
            beschreibung.append( " " );
            beschreibung.append( I18n.i18n( "TalentBonusFest.Leittalent" ) );
        }

        return beschreibung.toString();
    }

    /**
     * @return die Stufenboni die es auf die Talente zu verteilen gilt.
     */
    public final List< Integer > getBoni() {
        return new ArrayList< Integer >( stufenBoni );
    }

    public double getGpKosten() {

        double kosten = 0.0;

        for ( final int bonus : stufenBoni ) {

            for ( final Talent talent : talente.getTalente() ) {

                double kostenTalent = 0.0;

                if ( 0 < bonus ) {

                    if ( ! talent.istBasisTalent() ) {

                        kostenTalent += talent.getSteigerungsfaktor();
                    }

                    kostenTalent += ( bonus * talent.getSteigerungsfaktor() );

                    kostenTalent /= 10.0;

                } else {

                    kostenTalent += ( bonus * 0.5 );
                }

                kosten += kostenTalent;
            }
        }

        kosten /= talente.getTalente().size();

        return kosten;
    }

    /** {@inheritDoc} */
    public final Talent getTalent() {
        return talente.getTalente().get( 0 );
    }

    /**
     * Fuer den Fall dass der Talentbonus direkt anwendbar ist, koennen
     * hier die Talentboni ermittelt werden, die der Held erhaelt.
     *
     * @see #istDirektAnwendbar(Held)
     * @return die Talenboni die der Held erhaelt.
     */
    public List< TalentWert > getTalentboni( final Held held ) {

        final Talent talent = getTalente( held ).get( 0 );

        final int stufenbonus = stufenBoni.get( 0 );

        final TalentWert wert = new TalentWert( talent, stufenbonus );

        final List< TalentWert > talentboni = new ArrayList< TalentWert >( 1 );

        talentboni.add( wert );

        return talentboni;
    }

    /**
     * @param held der die Talentboni erhalten soll.
     * @return die Talente auf welche die Stufenboni verteilt werden sollen.
     */
    public final List< Talent > getTalente( final Held held ) {
        return new ArrayList< Talent >( talente.getTalente( held ) );
    }

    /** {@inheritDoc} */
    public final Talentgruppe getTalentgruppe() {
        return talente.getTalentgruppe();
    }

    /** @return die {@link TalentListe} welche die Talent liefert. */
    public final TalentListe getTalentListe() {
        return talente;
    }

    /** {@inheritDoc} */
    public final TalentbonusTyp getTyp() {
        return typ;
    }

    /**
     * @see #getTalentboni(Held)
     * @return {@code true} wenn der Bonus direkt anwendbar ist und es nicht
     *      noetig ist eine Auwahl zu treffen, sonst {@code false}.
     *      Die Talentboni die der Held erhaelt koenne in ersterem Fall ueber
     *      die Methode {@link #getTalentboni(Held)} abgerufen werden.
     */
    public boolean istDirektAnwendbar( final Held held ) {
        return ( 1 == getTalente( held ).size() );
    }

    /**
     * @return {@code true} wenn das durch den Bonus gesteigerte Talent zu
     *      einem Leittalent wird, sonst {@code false}.
     */
    public final boolean istLeittalent() {
        return leittalent;
    }

    /**
     * @param istLeittalent {@code true} wenn das durch den Bonus besteigerte
     *      Talent zu einem Leittalent wird, sonst {@code false}.
     */
    public final void setLeittalent( final boolean istLeittalent ) {
        
        leittalent = istLeittalent;
    }

    /**
     * Aendert den Typ des Talent-Bonus.
     *
     * @param neuerTyp fuer den Bonus.
     */
    public final void setTyp( final TalentbonusTyp neuerTyp ) {
        typ = neuerTyp;
    }
    /**
     * @return eine Beschreibung wie 'Singen, Tanzen oder Zechen +5'.
     */
    @Override
    public String toString() {
        
        return getBeschreibung();
    }

    /** {@inheritDoc} */
    public IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_FEST );

        element.addChild( talente.toXml() );

        for ( final int bonus : stufenBoni ) {

            final IXMLElement bonusElement = new XMLElement( TAG_BONUS );

            bonusElement.setContent( String.valueOf( bonus ) );

            element.addChild( bonusElement );
        }

        if ( leittalent ) {

            element.setAttribute( ATTR_LEITTALENT, "true" );
        }

        if ( typ.equals( TalentbonusTyp.ENTDECKER ) ) {

            element.setAttribute( ATTR_TYP, "Entdecker" );
        }

        return element;
    }
}
