/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.util;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_BIS;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_ERGEBNIS;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_VON;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_EINTRAG;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Wuerfel;
import de.bernhardjung.dsaprogramm.i18n.I18n;

public class Wuerfeltabelle {

    /**
     * Die Tabelle mit den Ergebnissen.
     * Sie ist 21 Felder gross, so dass den Indexpositionen von 1 bis 20 ein
     * Eintrag zugeordnet werden kann.
     */
    private final String[] tabelle;

    private final List< Tabelleneintrag > tabelleneintraege;

    public Wuerfeltabelle() {

        tabelle = new String[ 21 ];

        tabelleneintraege = new ArrayList< Tabelleneintrag >();
    }

    /**
     * Fuellt eine Zeile der HTML-Tabelle mit Tabelleneintraegen.
     *
     * @param tabelle
     *            Die HTML-Tabelle die es zu fuellen gilt.
     * @param von
     *            Der Index ab dem die {@link #tabelleneintraege} in die Tabelle
     *            eingefuegt werden.
     * @param bis
     *            Der Index bis zu dem (einschliesslich) die
     *            {@link #tabelleneintraege} in die Tabelle eingefuegt werden.
     */
    private void appendHorizontal(
            final StringBuilder tabelle,
            final int von,
            final int bis ) {

        tabelle.append( "<tr>" );
        for ( int i = von; i <= bis ; ++i ) {
            final Tabelleneintrag eintrag = tabelleneintraege.get( i );
            tabelle.append( "<td>" );
            tabelle.append( eintrag.von );
            if ( eintrag.von < eintrag.bis ) {
                tabelle.append( "-" );
                tabelle.append( eintrag.bis );
            }
            tabelle.append( "</td>" );
        }
        tabelle.append( "</tr>" );

        tabelle.append( "<tr>" );
        for ( int i = von; i <= bis; ++i ) {
            final Tabelleneintrag eintrag = tabelleneintraege.get( i );
            tabelle.append( "<td>" );
            tabelle.append( getElementAn( eintrag.von ) );
            tabelle.append( "</td>" );
        }
        tabelle.append( "</tr>" );

    }
    /**
     * Fuegt einen neune Eintrag in die Tabelle ein.
     *
     * @param von
     *            der kleinste Wert mit dem das Ergebnis erwuerfelt werden soll.
     *            Der kleinste erlaubte Wert ist die 1.
     * @param bis
     *            der groesste Wert mit dem das Ergebnis erwuerfelt werden soll.
     *            Der groesste erlaubte Wert ist die 20.
     * @param id
     *            das bei einem Wert von {@code von} bis {@code bis}, beides
     *            einschliesslich, geliefert werden soll. Es kann sich entweder
     *            direkt um einen verwendbaren Text handeln, oder um einen
     *            Schluessel fuer einen Text aus der Lokalisierungsdatei.
     * @throws IllegalArgumentException
     *             wenn ein Bereich angegeben wird, in dem schon Eintraege
     *             vorhanden sind. Wenn nicht {@code von &le; bis} gilt.
     * @throws IndexOutOfBoundsException
     *             wenn der angegebene Bereich ausserhalb der erlaubten Grenzen
     *             liegt.
     */
    public final void eintragen(
            final int von,
            final int bis,
            final String id ) {

        if ( von > bis ) {

            throw new IllegalArgumentException( String.format(
                    "von darf nicht groesser als bis sein.\n" +
                    "von = %1$s\n" +
                    "bis = %2$s",
                    von, bis ) );
        }

        if ( von < 1 || 20 < bis ) {

            throw new IndexOutOfBoundsException( String.format(
                    "Der Bereich %1$s bis %2$s liegt ausserhalb der " +
                    "erlaubten Grenzen von 1 bis 20.") );
        }

        for ( int i = von ; i <= bis ; ++i ) {

            if ( tabelle[ i ] != null ) {

                throw new IllegalArgumentException( String.format(
                        "Der Tabellenposition %1$s wurde schon ein Ergebnis " +
                        "zugeordnet.\n" +
                        "vorhandenes Ergebnis: %2$s\n" +
                        "      neues Ergebnis: %3$s",
                        i, tabelle[ i ], id ) );
            }

            tabelle[ i ] = id;
        }

        final Tabelleneintrag eintrag;
        eintrag = new Tabelleneintrag( von, bis, id );

        int index;
        index = Collections.binarySearch( tabelleneintraege, eintrag );

        index = -( index + 1 );

        tabelleneintraege.add( index, eintrag  );
    }

    /**
     * @return Die Tabelleneintraege der Tabelle.
     */
    public final List< Tabelleneintrag > getTabelleneintraege() {
        return new ArrayList< Tabelleneintrag >( tabelleneintraege );
    }

    /**
     * Liefert das Element aus der angegebenen Zeile der Tabelle.
     * @param zeile ein Wert von 1 bis 20.
     * @return
     */
    private String getElementAn( int zeile ) {
        
        final String element = tabelle[ zeile ];
        
        if ( I18n.hatLokalisierungstext( element ) ) {
            
            return I18n.i18n( element );
        
        } else {
            
            return element;
        }
    }
    
    /**
     * @return einen Zufaellig gewaehlten Wert aus der Tabelle.
     */
    public final String wuerfeln() {
        return getElementAn( Wuerfel.W20.wuerfeln() );
    }

    /**
     * Liefert eine horizontal ausgerichtete HTML-Tabelle.
     * Beispiel:
     * <table border="1">
     * <tr><td>1-3</td><td>4-7</td><td>...</td></tr>
     * <tr><td>schwarz</td><td>braun</td><td>...</td></tr>
     * </table>
     * <strong>Der gelieferte String enthaelt noch kein oeffnendes HTML-Tag
     * </strong>
     * @return eine HTML Repraesentation der Tabelle.
     */
    public final String toHtmlTabelleHorizontal() {

        final StringBuilder tabelle;
        tabelle = new StringBuilder( "<table width=\"100%\">" );

        final int size = tabelleneintraege.size();

        final int bis = Math.min( size-1, 3 );

        appendHorizontal( tabelle, 0, bis );

        if ( 4 < size ) {

            appendHorizontal( tabelle, 4, size-1 );
        }

        tabelle.append( "</table>" );

        return tabelle.toString();
    }

    /**
     * Liefert eine vertikal ausgerichtete HTML-Tabelle.
     * Beispiel:
     * <table>
     * <tr><td>1-3</td><td>schwarz</td></tr>
     * <tr><td>4-7</td><td>braun</td></tr>
     * <tr><td>...</td><td>...</td></tr>
     * </table>
     * <strong>Der gelieferte String enthaelt noch kein oeffnendes HTML-Tag
     * </strong>
     * @return eine HTML Repraesentation der Tabelle.
     */
    public final String toHtmlTabelleVertikal() {

        final StringBuilder tabelle = new StringBuilder( "<table>" );

        for ( final Tabelleneintrag eintrag : tabelleneintraege ) {

            tabelle.append( eintrag.toHtmlTabellenZeile() );
        }

        tabelle.append( "</table>" );

        return tabelle.toString();
    }

    /**
     * @return ein XML-Element, das die Eintraege der Wuerfeltabelle enthaelt.
     */
    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( "Wuerfeltabelle" );

        for ( final Tabelleneintrag eintrag : tabelleneintraege ) {

            final IXMLElement eintragElement = new XMLElement( TAG_EINTRAG );

            eintragElement.setAttribute( ATTR_VON,
                                         String.valueOf( eintrag.von ) );

            eintragElement.setAttribute( ATTR_BIS,
                                         String.valueOf( eintrag.bis ) );

            eintragElement.setAttribute( ATTR_ERGEBNIS, eintrag.id );

            element.addChild( eintragElement );
        }

        return element;
    }

    /**
     * Ein Eintrag aus der Wuerfeltabelle.
     * Es wird ein Intervall angegeben und das mit dem Intervall
     * verbundene Ergebnis.
     *
     * @author Bernhard Jung
     *
     */
    public static class Tabelleneintrag implements Comparable< Tabelleneintrag > {

        /** Untergrenze Intervall (einschliesslich). */
        public final int von;

        /** Obergrenze Intervall (einschliesslich). */
        public final int bis;

        /**
         * Das zu erwuerfelnde Ergebnis oder ein Schluessel fuer einen Text aus
         * der Lokalisierungsdatei.
         */
        public final String id;

        /** Der Eintrag der Wuerfeltabelle als HTML-Tabellenzeile. */
        private String html;

        /**
         * Erzeugt einen neuen Eintrag in einer Wuerfeltabelle. Dazu wird ein
         * Intervall angegeben und das Ergebnis, das in diesem Intervall
         * erwuerfelt werden kann.
         *
         * @param von
         *            Untergrenze Intervall (einschliessliche)
         * @param bis
         *            Obergrenze Intervall (einschliesslich)
         * @param id
         *            Wieder das Ergebnis, oder ein Schluessel unter dem das
         *            Ergebnis in der Lokalisierungsdatei zu finden ist.
         */
        public Tabelleneintrag(
                final int von,
                final int bis,
                final String id ) {

            this.von = von;
            this.bis = bis;
            this.id = id;
        }

        public final int compareTo(final Tabelleneintrag o) {
            return ( von - o.von );
        }

        public final String toHtmlTabellenZeile() {

            if ( null == html ) {

                final StringBuilder builder = new StringBuilder();

                builder.append( "<tr><td align=\"right\">" );
                builder.append( von );
                if ( von < bis ) {
                    builder.append( "-" );
                    builder.append( bis );
                }
                builder.append( "</td><td>" );
                builder.append( getErgebnis() );
                builder.append( "</td></tr>" );

                html = builder.toString();
            }

            return html;
        }

        @Override
        public final boolean equals( final Object o ) {

            if ( null == o ) {
                return false;
            }

            if ( o.getClass() == getClass() ) {

                final Tabelleneintrag anderer = (Tabelleneintrag) o;

                return ( von == anderer.von )
                    && ( bis == anderer.bis )
                    && ( id.equals( anderer.id ) );

            } else {

                return false;

            }
        }
        
        private String getErgebnis() {
            
            if ( I18n.hatLokalisierungstext( id ) ) {
                
                return I18n.i18n( id );
            
            } else {
                
                return id;
            }
        }

        @Override
        public final String toString() {
            return String.format( "%1$s-%2$s: %3$s", von, bis, id );
        }
    }
}
