/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten.factorys;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import net.n3.nanoxml.IXMLElement;

public abstract class Factory< T > {

    /**
     * Erzeugt die Elemente aus dem XML-Dokument.
     *
     * @param wurzelElement
     *            des XML-Dokuments.
     * @return die erzeugten Elemente.
     */
    public abstract Collection< T > erzeugen( final IXMLElement wurzelElement );

    /**
     * Liest ein XML-Attribut aus, das einen Boolean Wert enthaelt.
     *
     * @param element
     *            dessen Attribut ausgelesen werden soll.
     * @param attribut
     *            das den Boolean Wert enthaelt.
     * @return die Boolean Wert.
     */
    public final boolean getBoolean(
            final IXMLElement element,
            final String attribut ) {

        final String string = element.getAttribute( attribut );

        return Boolean.valueOf( string );
    }

    /**
     * Liest ein XML-Attribut aus, das einen Integer Wert enthaelt.
     *
     * @param element
     *            dessen Attribut ausgelesen werden soll.
     * @param attribut
     *            das den Integer Wert enthaelt.
     * @return die Integer Wert.
     */
    public final int getInt(
            final IXMLElement element,
            final String attribut ) {

        final String string = element.getAttribute( attribut );

        return Integer.valueOf( string );
    }

    /**
     * Liest ein XML-Attribut aus, das eine Liste von Integer-Werten enthaelt.
     * Die Zahlen sind als mit Komma getrennte Liste gespeichert.
     * Beispiel: "1,-2,3,-4,5".
     *
     * @param element
     *            dessen Attribut ausgelesen werden soll.
     * @param attribut
     *            das ausgelesen werden soll.
     * @return die Liste der Integer-Werte.
     */
    public final List< Integer > getIntListe(
            final IXMLElement element,
            final String attribut ) {

        final String[] werte = element.getAttribute( attribut ).split( "," );

        final List< Integer > liste = new ArrayList< Integer >( werte.length );

        for ( final String wert : werte ) {

            liste.add( Integer.valueOf( wert ) );
        }

        return liste;
    }
}
