/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import de.bernhardjung.dsaprogramm.basis.GpNachteile;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.eigenschaften.EigenschaftWert;
import de.bernhardjung.dsaprogramm.erschaffung.ErschaffungManager;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;

class PanelProbleme extends JPanel implements Observer, Uebersetzbar {

    private static final String HTML_START = "<html><body style=\"padding:10px;\">";

    private static final String GP_SCHLECHTE_EIGENSCHAFTEN_HINWEIS = 
        "PanelProbleme.GpSchlechteEigenschaftenHinweis";

    private static final String GP_NACHTEILE_HINWEIS = 
        "PanelProbleme.GpNachteileHinweis";

    private static final String ZU_VIELE_ZAUER_AKTIVIERT = 
        "PanelProbleme.MaximalAktivierbar";

    private static final String ZU_VIELE_TALENTE_AKTIVIERT = 
        "PanelProbleme.ZuVieleAktiviert";

    /** Teilt dem Benutzer mit, dass das AP-Konto ueberzogen ist. */
    public static final String AP_UEBERZOGEN =
        "PanelProbleme.ApUeberzogen";

    /** Die Ueberschrift fuer die Eigenschaften. */
    private static final String EIGENSCHAFTEN =
        "PanelProbleme.Eigenschaften";

    /** Teilt dem Benutzer mit, dass das GP-Konto ueberzogen ist. */
    public static final String GP_UEBERZOGEN =
        "PanelProbleme.GpUeberzogen";

    /**
     * Der Text der angezeigt wird wenn es in einer Kategorie keine Probleme
     * gibt.
     */
    private static final String KEINE_PROBLEME =
        "PanelProbleme.KeineProbleme";

    /** Die Ueberschrift fuer die Kultur-Probleme. */
    private static final String KULTUR =
        "PanelProbleme.Kultur";

    /** Die Ueberschrift fuer die Professions-Probleme. */
    private static final String PROFESSION =
        "PanelProbleme.Profession";

    /** Die Ueberschrift fuer die Rassen-Probleme. */
    private static final String RASSE =
        "PanelProbleme.Rasse";

    /** Die Ueberschrift fuer die Sonderfertigkeiten. */
    private static final String SONDERFERTIGKEITEN =
        "PanelProbleme.Sonderfertigkeiten";

    /**
     * Der Hinweistext zu den Regeln fuer die Voraussetzungen von
     * Sonderfertigkeiten.
     */
    private static final String SONDERFERTIGKEITEN_HINWEIS =
        "PanelProbleme.Sonderfertigkeiten.Hinweis";

    /** Die Ueberschrift fuer die Talente. */
    private static final String TALENTE =
        "PanelProbleme.Talente";

    /** Die Ueberschrift fuer die Vor- und Nachteile. */
    private static final String VOR_NACHTEILE =
        "PanelProbleme.VorNachteile";

    /** Die Ueberschrift fuer die Zauber. */
    private static final String ZAUBER =
        "PanelProbleme.Zauber";

    /** Hier werden die Probleme angezeigt. */
    private final JEditorPane editorPane;

    /** Der Manager fuer die aktuelle Heldenerschaffung. */
    private ErschaffungManager manager;

    PanelProbleme() {

        editorPane = new JEditorPane();

        editorPane.setEditable( false );

        editorPane.setContentType( "text/html" );

        editorPane.setSize( new Dimension( 10, 10 ) );

        final JScrollPane scrollPane = new JScrollPane( editorPane );

        scrollPane.setPreferredSize( new Dimension( 10, 10 ) );

        setLayout( new BorderLayout() );

        add( scrollPane, BorderLayout.CENTER );
    }

    public void neuUebersetzen() {
        
        update( null, null );
    }
    
    /**
     * Registiert einen Held bei dem Panel.
     *
     * @param derManager fuer die aktuelle Heldenerschaffung.
     */
    public final void setManager( final ErschaffungManager derManager ) {

        manager = derManager;

        manager.addObserver( this );

        final Held held = manager.getHeld();

        held.addObserver( this );

        update( null, null );
    }

    /**
     * Haengt, wenn es Probleme gibt, eine Beschreibung der Regelkonflikte, als
     * Definition-List (&lt;dl&gt;), an einen StringBuilder an.
     *
     * @param text
     *            der die gesammelten Regelkonflikte anzeigt.
     * @param wert
     *            dessen Regelkonflikte angezeigt werden sollen.
     * @param beschreibung
     *            der Regekonflikte die in Zusammenhang mit {@code wert}
     *            auftreten.
     */
    private void append(
            final StringBuilder text,
            final String wert,
            final KonfliktBeschreibung beschreibung ) {

        if ( beschreibung.enthaeltKonflikt() ) {

            text.append( "<dl><dt><b>" );
            text.append( wert );
            text.append( ":</b></dt><dd>" );
            text.append( beschreibung.toHtml2() );
            text.append( "</dd></dl>" );
        }
    }

    /**
     * Haengt eine Problembeschreibung an einen StringBuilder an.
     *
     * @param text
     *            an den die Problembeschreibung angehaengt werden soll.
     * @param ueberschrift
     *            fuer die Problembeschreibung.
     * @param probleme
     *            die in die Problembeschreibung eingefuegt werden sollen.
     * @return {@code true}, wenn eine Problembeschreibung an den text
     *         angehaengt wurde oder {@code false} wenn es keine Probleme gab,
     *         die an den Text angehaengt werden mussten.
     */
    private boolean append(
            final StringBuilder text,
            final String ueberschrift,
            final StringBuilder probleme ) {

        if ( 0 < probleme.length() ) {

            ueberschrift( text, ueberschrift );

            text.append( probleme );

            return true;
        }

        return false;
    }

    /**
     * Haengt eine Ueberschrift an einen StringBuilder an.
     *
     * @param text an den die Ueberschrift angehaengt werden soll.
     * @param ueberschrift Der Text fuer die Ueberschrift.
     */
    private void ueberschrift(
            final StringBuilder text,
            final String ueberschrift ) {

        text.append( "<b><u><font size=\"+1\">" );
        text.append( ueberschrift );
        text.append( "</font></u></b><br>" );
    }

    /**
     * @param o ungenutzt.
     * @param arg ungenutzt;
     */
    public final void update( final Observable o, final Object arg ) {

        final Held held = manager.getHeld();

        final StringBuilder text = new StringBuilder( HTML_START );

        final StringBuilder probleme = new StringBuilder();

        // AP und GP
        // *********
        final int gp = manager.getGpGuthaben();
        if ( gp < 0 ) {
            text.append( "<b><font size=\"+1\">" );
            text.append( I18n.i18n( GP_UEBERZOGEN ) );
            text.append( "</font></b><br><br>" );
        }

        final int ap = manager.getApGuthaben();
        final int apZauber = manager.getApGuthabenMagie();

        if ( ( ap < 0 ) || ( apZauber < 0 ) ) {
            text.append( "<b><font size=\"+1\">" );
            text.append( I18n.i18n( AP_UEBERZOGEN ) );
            text.append( "</font></b><br><br>" );
        }

        // Rasse
        // *********
        final Rasse rasse = held.getRasse();

        append( probleme, rasse.getName(), rasse.pruefen( held ) );

        append( text, I18n.i18n( RASSE ), probleme );

        probleme.setLength( 0 );

        // Kultur
        // **********
        final Kultur kultur = held.getKultur();

        append( probleme, kultur.getName(), kultur.pruefen( held ) );

        append( text, I18n.i18n( KULTUR ), probleme );

        probleme.setLength( 0 );

        // Profession
        // **************
        final Profession profession = held.getProfession();

        append( probleme, profession.getName(), profession.pruefen( held ) );

        append( text, I18n.i18n( PROFESSION ), probleme );

        probleme.setLength( 0 );

        // Eigenschaften
        // *****************
        for ( final Eigenschaft eigenschaft :
                                        Eigenschaft.getEigenschaftenMitSo() ) {

            final EigenschaftWert wert = held.getWert( eigenschaft );

            append( probleme, eigenschaft.getName(), wert.pruefen() );
        }

        append( text, I18n.i18n( EIGENSCHAFTEN ), probleme );

        probleme.setLength( 0 );

        // Talente
        // ***********
        final int anzahlAktivierteTalente = held.getAnzahlAktivierteTalente();

        if ( anzahlAktivierteTalente > PanelTalente.MAX_AKTIVIEREN ) {

            probleme.append( I18n.i18n( ZU_VIELE_TALENTE_AKTIVIERT ) );
            probleme.append( "<br>" );
        }

        for ( final Talent talent : held.getTalente() ) {

            append( probleme, talent.getName(), talent.pruefen( held ) );
        }

        for ( final Talent talent : held.getFehlendeTalente() ) {

            append( probleme, talent.getName(), talent.pruefen( held ) );
        }

        append( text, I18n.i18n( TALENTE ), probleme );

        probleme.setLength( 0 );

        // Zauber
        // **********
        final int anzahlAktivierteZauber = held.getAnzahlAktivierteZauber();

        if ( anzahlAktivierteZauber > PanelZauber.MAX_AKTIVIEREN ) {

            probleme.append( String.format( 
                    I18n.i18n( ZU_VIELE_ZAUER_AKTIVIERT ),
                    PanelZauber.MAX_AKTIVIEREN ) );
            probleme.append( "<br>" );
        }

        for ( final ZauberWert wert : held.getZauber() ) {

            append( probleme, wert.getName(), wert.pruefen( held ) );
        }

        for ( final ZauberWert wert : held.getFehlendeZauber() ) {

            append( probleme, wert.getName(), wert.pruefen( held ) );
        }

        append( text, I18n.i18n( ZAUBER ), probleme );

        probleme.setLength( 0 );

        // Sonderfertigkeiten
        // **********************
        for ( final SonderfertigkeitWert wert : held.getSonderfertigkeiten() ) {

            append( probleme, wert.getName(), wert.pruefen( held ) );
        }

        for ( final SonderfertigkeitWert wert :
                                        held.getFehlendeSonderfertigkeiten() ) {

            append( probleme, wert.getName(), wert.pruefen( held ) );
        }

        if ( append( text, I18n.i18n( SONDERFERTIGKEITEN ), probleme ) ) {

            text.append( "<p>" );
            text.append( I18n.i18n( SONDERFERTIGKEITEN_HINWEIS ) );
            text.append( "</p>" );
        }

        probleme.setLength( 0 );

        // Vor-/Nachteile
        // ******************
        final GpNachteile gpNachteile = held.getGpNachteile();
        if ( PanelVorNachteile.MAX_GP_NACHTEILE < gpNachteile.nachteile ) {

            probleme.append( "<p>" );
            probleme.append( I18n.i18n( GP_NACHTEILE_HINWEIS ) );
            probleme.append( "</p>" );
        }
        if ( PanelVorNachteile.MAX_GP_SCHLECHTE_EIGENSCHAFTEN
                < gpNachteile.schlechteEigenschaften ) {

            probleme.append( "<p>" );
            probleme.append( I18n.i18n( GP_SCHLECHTE_EIGENSCHAFTEN_HINWEIS ) );
            probleme.append( "</p>" );
        }

        for ( final VorNachteilWert wert : held.getVorNachteile() ) {

            append( probleme, wert.getName(), wert.pruefen( held ) );
        }

        for ( final VorNachteilWert wert : held.getFehlendeVorNachteile() ) {

            append( probleme, wert.getName(), wert.pruefen( held ) );
        }

        append( text, I18n.i18n( VOR_NACHTEILE ), probleme );

        probleme.setLength( 0 );

        // Pruefen ob es Probleme gab.
        if ( text.length() == HTML_START.length() ) {

            text.append( "<h1>" );
            text.append( I18n.i18n( KEINE_PROBLEME ) );
            text.append( "</h1>" );
        }

        text.append( "</body></html>" );

        editorPane.setText( text.toString() );

        editorPane.setCaretPosition( 0 );
    }
}
