/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.SKT;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.TalentWert;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

class PanelTalenteListe
extends ResetbaresPanel
implements Observer, Uebersetzbar {

    /** Ein String um die Aktivierungs-/Steigerungskosten anzuzeigen. */
    static final String AP = "PanelTalenteListe.AP";

    /** Das Icon das neben einem Leittalent angezeigt wird. */
    static final ImageIcon ICON_LEITTALENT = Util.ladeIcon( "leit" );

    /** Die Ueberschrift fuer die Spalte mit den Steigerungskosten. */
    static final String KOSTEN = "PanelTalenteListe.Kosten";

    /**
     * Die Ueberschrift fuer die Spalte mit den Steigerungsfaktoren der
     * Talente.
     */
    static final String FAKTOR = "PanelTalenteListe.Steigerungsfaktor";

    /** Die Ueberschrift fuer die Spalte mit den Proben. */
    private static final String PROBE  = "PanelTalenteListe.Probe";

    /** Die Ueberschrift fuer die Spalte mit der Speziellen Erfahrung. */
    static final String SE = "PanelTalenteListe.SpezielleErfahrung";

    /** Die Ueberschrift fuer die Spalte mit den Talenten. */
    static final String TALENT = "PanelTalenteListe.Talent";

    /** Die Ueberschrift fuer die Spalte mit den Talentstufen. */
    static final String TAW    = "PanelTalenteListe.TaW";

    /** Der Text fuer das Icon, das ein Leittalent markiert. */
    static final String TEXT_LEITTALENT = "PanelTalenteListe.Leittalent";

    /**
     * Speichert Regelemente wie eine Eigenschaft oder Talent als Schluessel.
     * Die Liste enthaelt Indizes fuer Talente aus {@link #talente}.
     * Die Indizes geben die Talente an die von dem Regelelement abhaengig sind.
     * Wenn ein Schluessel, z.B. eine Eigenschaft, geaendert wird, muss
     * die Anzeige fuer alle davon abhaengigen Talente aktualisiert werden.
     */
    private final Map< Object, List< Integer > > abhaengigkeiten;

    /**
     * Die JCheckBoxen die in der ersten Spalte stehen und mit denen angezeigt
     * wird ob das Talent aktiviert ist.
     * Die i-te Checkbox in der Oberflaeche (von oben gezaehlt) steht an
     * Listen-Position i.
     */
    private final List< JCheckBox > aktiviert;

    /** Eine fette (bold) Schrift. */
    private final Font fetteSchrift;

    /** Der Held dessen Wert angezeigt und manipuliert werden. */
    private Held held;

    /**
     * Die JLabel mit denen die Steigerungskosten angezeigt werden.
     * Das i-te Label in der Oberflaeche (von oben gezaehlt) steht an
     * Listen-Position i.
     */
    private final List< JLabel > kosten;
    
    /** Die kleine Icons, die ein Leittalent markieren. */
    private final List< JLabel > leittalente;

    /** Liefert die Lernmethode zum Steigern der Talente. */
    private final Lernmethoden lernmethoden;

    /** Die Label, die die Talentproben anzeigen. */
    private final List< JLabel > proben;
    
    
    
    /**
     * Enthaelt die Checkboxen mit denen eine spezielle Erfahrung angegeben
     * werden kann.
     */
    private final List< JCheckBox > spezielleErfahrung;
    
    /**
     * {@code true}, wenn das Panel beim Steigern des Helden benutzt wird,
     * {@code false}, wenn das Panel bei der Charaktererschaffung benutzt wird.
     */
    private final boolean steigern;

    /**
     * Die JLabel die die Steigerungsfaktoren der Talente anzeigen.
     * Das i-te Label in der Oberflaeche (von oben gezaehlt) steht an
     * Listen-Position i.
     */
    private final List< JLabel > steigerungsfaktoren;
    
    /**
     * Die JSpinner mit denen die Stufen der Talente angezeigt und manipuliert
     * werden.
     * Der Spinner an Listen-Position i steht in der Oberflaeche an i-ter
     * Stelle (von oben gezaehlt).
     */
    private final List< JSpinner > stufen;

    /**
     * Die angezeigten Talente.
     * Das Talent an Position i wird in der Benutzeroberflaech in Zeile i
     * angezeigt. Die Zeilen mit den Trennstrichen nicht mitgerechnet.
     */
    private final Talentgruppe talente;
    
    /** Uebersetzt die Ueberschriften der Liste. */
    private final Uebersetzer uebersetzer;
    
    /**
     * Erzeugt ein neues Panel.
     *
     * @param dieTalente
     *            die im Panel dargestellt werden sollen
     * @param zumSteigern
     *            {@code true}, wenn das Panel beim Steigern des Helden benutzt
     *            wird, {@code false}, wenn das Panel bei der
     *            Charaktererschaffung benutzt wird.
     * @param dieLernmethoden
     *            ist {@code zumSteigern == true} muss hier ein Objekt
     *            uebergeben werden, das bestimmt nach welcher Lernmethode die
     *            Talente gesteigert werden. Ist {@code zumSteigern == false},
     *            kann hier {@code null} uebergeben werden.
     */
    PanelTalenteListe(
            final Talentgruppe dieTalente,
            final boolean zumSteigern,
            final Lernmethoden dieLernmethoden ) {

        super( new BorderLayout() );

        uebersetzer = new Uebersetzer();
        
        
        
        talente = dieTalente;

        abhaengigkeiten = new HashMap< Object, List< Integer > >();

        initAbhaengigkeiten();

        aktiviert = new ArrayList< JCheckBox >( talente.size() );

        fetteSchrift = getFont().deriveFont( Font.BOLD );

        kosten = new ArrayList< JLabel >( talente.size() );
      
        leittalente = new ArrayList< JLabel >( talente.size() );

        lernmethoden = dieLernmethoden;
        if ( zumSteigern ) {
            lernmethoden.addObserver( this );
        }

        proben = new ArrayList< JLabel >( talente.size() );
        
        spezielleErfahrung = new ArrayList< JCheckBox >( talente.size() );

        steigern = zumSteigern;

        steigerungsfaktoren = new ArrayList< JLabel >( talente.size() );

        stufen = new ArrayList< JSpinner >( talente.size() );

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Baut die Benutzeroberflaeche auf.
     */
    private void aufbauenBenutzeroberflaeche() {
        final ActionListener actionDeAktivieren;
        final ChangeListener actionStufe;

        if ( steigern ) {

            actionDeAktivieren = new ActionDeAktivierenSteigern();

            actionStufe = new ActionStufeSteigern();

        } else {

            actionDeAktivieren = new ActionDeAktivierenErschaffung();

            actionStufe = new ActionStufeErschaffung();

        }

        final ActionListener actionSpezielleErfahrung = new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                final JCheckBox checkBox = (JCheckBox) e.getSource();

                final int index = Integer.valueOf( checkBox.getName() );

                final Talent talent = talente.get( index );

                if ( held.hat( talent ) ) {

                    final TalentWert wert = held.getWert( talent );

                    wert.setSpezielleErfahrung( checkBox.isSelected() );
                }
            }
        };

        final JPanel panel = new JPanel();

        panel.setLayout( erzeugeLayout() );

        final CellConstraints cc = new CellConstraints();

        int x = 2;
        panel.add( 
                uebersetzer.erzeugeUeberschrift( TALENT ), 
                cc.xy( x += 2, 2 ) );
        
        panel.add( 
                uebersetzer.erzeugeUeberschrift( FAKTOR ), 
                cc.xy( x += 2, 2 ) );
        
        panel.add( 
                uebersetzer.erzeugeUeberschrift( PROBE ),  
                cc.xy( x += 2, 2 ) );
        
        panel.add( 
                uebersetzer.erzeugeUeberschrift( TAW ),  
                cc.xy( x += 2, 2 ) );
        
        panel.add( 
                uebersetzer.erzeugeUeberschrift( KOSTEN ), 
                cc.xy( x += 2, 2 ) );
        
        if ( steigern ) {
            panel.add( 
                    uebersetzer.erzeugeUeberschrift( SE ), 
                    cc.xy( x += 2, 2 ) );
        }

        // In der 2. und ab da in jeder 5. Zeile wird ein Trennstrich
        // eingefuegt.
        final int extraZeilen = 1 + ( talente.size() / 4 );
        // trennstriche gibt an wie viele Trennstriche schon eingefuegt wurden.
        for ( int i = 0, n = talente.size() + extraZeilen, trennstriche = 0
                ; i < n
                ; ++i ) {

            x = 2;
            final int y = ( 2 * i ) + 4;

            // alle 5 Zeilen ein Trennstrich.
            if ( ( i % 5 ) == 0 ) {

                panel.add( new JSeparator(), cc.xyw( x, y, 14 ) );

                ++trennstriche;

                continue;
            }

            final JLabel leittalent = new JLabel();

            leittalente.add( leittalent );

            panel.add( leittalent, cc.xy( x, y ) );

            final Talent talent = talente.get( i - trennstriche );

            // Die Checkbox zum (De)Aktivieren
            // *******************************
            final JCheckBox checkBox = new JCheckBox( talent.getName() );

            checkBox.addActionListener( actionDeAktivieren );

            // Jede Checkbox kennt die Zeile in der sie steht und damit auch
            // das Talent fuer das sie da ist.
            final String index = String.valueOf( i - trennstriche );

            checkBox.setName( index );

            aktiviert.add( checkBox );

            x += 2;
            panel.add( checkBox, cc.xy( x, y ) );

            // Der Steigerungsfaktor
            // *********************
            final int faktor = talent.getSteigerungsspalte();

            final JLabel faktorLabel = new JLabel( SKT.toString( faktor ) );

            steigerungsfaktoren.add( faktorLabel );

            x += 2;
            panel.add( faktorLabel, cc.xy( x, y ) );

            // Die Probe
            // *********
            final String probeString = talent.getProbe().toString();

            final JLabel probe = new JLabel( probeString );
            
            proben.add( probe );
            
            x += 2;
            panel.add( probe, cc.xy( x, y ) );

            // Spinner fuer die Stufe
            // **********************
            final JSpinner spinner = Util.erzeugeSpinner( 0, -99, 99, 1 );

            spinner.addChangeListener( actionStufe );

            stufen.add( spinner );

            spinner.setName( index );

            x += 2;
            panel.add( spinner, cc.xy( x, y ) );

            // Die Kosten
            // **********
            final JLabel k = new JLabel();

            kosten.add( k );

            x += 2;
            panel.add( k, cc.xy( x, y ) );

            // Die Checkbox fuer die Spezielle Erfahrung
            // *****************************************
            if ( steigern ) {

                final JCheckBox se = new JCheckBox();

                se.setName( index );

                se.addActionListener( actionSpezielleErfahrung );

                spezielleErfahrung.add( se );

                x += 2;
                panel.add( se, cc.xy( x, y ) );
            }
        }

        final JScrollPane scrollPane = Util.erzeugeScrollPane( panel );

        resetter.einfuegen( scrollPane );
        
        add( scrollPane, BorderLayout.CENTER );
    }

    /**
     * @return das Layout fuer die Benutzeroberflaeche.
     */
    private FormLayout erzeugeLayout() {
        final String spaltenLayout;
        //                 Leittalent  Talent     Faktor    Probe     Stufe     Kosten
        spaltenLayout = "7dlu, P, 4dlu, F:P, 4dlu, C:P, 4dlu, P, 4dlu, C:P, 4dlu, P, " +
        //             SE
                "4dlu, P, 7dlu";

        // Jetzt das Zeilenlayout festlegen.
        // Es gibt eine Zeile fuer die Ueberschrift
        // eine Zeile mit einem Trennstrich unter der Ueberschrift.
        // eine Zeile fuer jedes Talent
        // und alle 5 Zeilen wird ein Trennstrich eingefuegt.
        final int extraZeilen = 1 + ( talente.size() / 4 );

        final StringBuilder zeilenLayout = new StringBuilder();

        zeilenLayout.append( "7dlu, P," );

        for ( int i = 0, n = talente.size() + extraZeilen ; i < n ; ++i ) {

            zeilenLayout.append( " 4dlu, P," );
        }

        zeilenLayout.append( " 7dlu" );

        return new FormLayout( spaltenLayout,
                               zeilenLayout.toString() );

    }

    /**
     * Baut {@link #abhaengigkeiten} auf.
     */
    private void initAbhaengigkeiten() {

        abhaengigkeiten.clear();
            
        for ( int i = 0, n = talente.size(); i < n ; ++i ) {

            final Talent talent = talente.get( i );

            for( final Object regelelement : talent.getAbhaengigkeiten() ) {

                setAbhaengigkeit( regelelement, i );
            }
        }
    }

    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
        
        initAbhaengigkeiten();
        
        for ( int i = talente.size()-1; i >= 0 ; --i ) {

            final Talent talent = talente.get( i );
            
            aktiviert.get( i ).setText( talent.getName() );
            
            if ( held.hatLeittalent( talent ) ) {

                leittalente.get( i ).setIcon( ICON_LEITTALENT );
                leittalente.get( i ).setToolTipText( I18n.i18n( 
                        TEXT_LEITTALENT ) );

            } else {

                leittalente.get( i ).setIcon( null );
                leittalente.get( i ).setToolTipText( null );
            }
        
            proben.get( i ).setText( talent.getProbe().toString() );
            
            update( i );
        }
    }
    
    /**
     * Legt fest, dass ein Talent von einem bestimmten Regelelement abhaengig
     * ist.
     *
     * @param regelelement
     *            von dem das Talent abhaengig ist.
     * @param index
     *            des Talents in {@link #talente}.
     */
    private void setAbhaengigkeit( final Object regelelement, final int index ) {

        if ( ! abhaengigkeiten.containsKey( regelelement ) ) {

            abhaengigkeiten.put( regelelement, new ArrayList< Integer >() );
        }

        abhaengigkeiten.get( regelelement ).add( index );
    }

    /**
     * Bestimmt den Helden dessen Werte im Panel angezeigt und editiert werden.
     *
     * @param derHeld der vom Panle bearbeitet wird.
     */
    public final void setHeld( final Held derHeld ) {

        held = derHeld;

        held.addObserver( this );

        for ( int i = 0, n = talente.size(); i < n ; ++i ) {

            if ( held.hatLeittalent( talente.get( i ) ) ) {

                leittalente.get( i ).setIcon( ICON_LEITTALENT );
                leittalente.get( i ).setToolTipText( I18n.i18n( 
                        TEXT_LEITTALENT ) );

            } else {

                leittalente.get( i ).setIcon( null );
                leittalente.get( i ).setToolTipText( null );

            }

            update( i );
        }
    }

    /**
     * Aktualisiert eine Zeile mit einem Talent.
     *
     * @param index des Talents, dessen Zeile aktualisiert werden soll.
     */
    private void update( final int index ) {

        final Talent talent = talente.get( index );

        final boolean hatTalent = held.hat( talent );

        final JCheckBox checkBox = aktiviert.get( index );

        // Aktiviert
        // *********
        if ( hatTalent ) {

            final TalentWert wert = held.getWert( talent );

            checkBox.setSelected( wert.istAktiviert() );

        } else {

            checkBox.setSelected( false );

        }


        // Konflikte
        // *********
        final KonfliktBeschreibung beschreibung = talent.pruefen( held );

        if ( beschreibung.enthaeltKonflikt() ) {

            // Unter Java 5 mit GTK L&F funktioniert das hier nicht, wenn im
            // naechsten Schritt die Schrift auf fett gestellt wird.
            // Die fette Schrift erscheint nicht farbig.
            checkBox.setForeground( Util.FEHLERFARBE );
            checkBox.setToolTipText( beschreibung.toHtml() );

        } else {

            checkBox.setForeground( getForeground() );
            checkBox.setToolTipText( null );

        }

        // Vorgegeben?
        // ***********
        if ( held.hatVorgegeben( talent ) ) {

            checkBox.setFont( fetteSchrift );

        } else {

            checkBox.setFont( getFont() );

        }

        // Steigerungsfaktor
        // *****************
        int stufe;

        if ( hatTalent ) {

            stufe = held.getStufe( talent );

        } else {

            stufe = 0;

        }

        final Lernmethode lernmethode;

        if ( steigern && ( null != lernmethoden.getLernmethode() ) ) {

            lernmethode = lernmethoden.getLernmethode();

        } else {

            lernmethode = Lernmethode.KEINE;
        }

        final int faktor = talent.getSteigerungsspalte(
                held,
                stufe,
                lernmethode );

        steigerungsfaktoren.get( index ).setText( SKT.toString( faktor ) );

        // Stufe
        // *****
        final JSpinner spinner = stufen.get( index );

        if ( hatTalent ) {

            stufe = held.getStufe( talent );

        } else {

            if ( held.hatVorgegeben( talent ) ) {

                stufe = held.getMindeststufe( talent );

            } else {

                stufe = 0;

            }
        }

        spinner.setValue( stufe );

        spinner.setEnabled( hatTalent );


        // Kosten
        // ******
        final int apKosten;
        if ( hatTalent && held.getWert( talent ).istAktiviert() ) {

            final TalentWert wert = held.getWert( talent );

            apKosten = wert.getSteigerungskosten( held, lernmethode );

        } else {

            apKosten = talent.getAktivierungskosten( held, lernmethode );
        }

        kosten.get( index ).setText( String.format( 
                I18n.i18n( AP ), 
                apKosten ) );

        // Spezielle Erfahrung?
        // ********************
        if ( steigern ) {

            final JCheckBox se = spezielleErfahrung.get( index );

            if ( hatTalent ) {

                se.setEnabled( true );

                se.setSelected( held.getWert( talent ).hatSpezielleErfahrung());

            } else {

                se.setEnabled( false );

                se.setSelected( false );
            }
        }
    }

    /**
     * Aktualisert die Anzeige wenn eine Veraenderung am Helden durchgefuehrt
     * wurde.
     *
     * @param o der ueberwachte Held.
     * @param arg Das Regelelement des Helden, das veraendert wurde.
     */
    public final void update( final Observable o, final Object arg ) {

        if ( Lernmethoden.LERNMETHODE.equals( arg ) ) {

            for ( int i = 0, n = talente.size() ; i < n ; ++i ) {

                update( i );
            }

        } else if ( abhaengigkeiten.containsKey( arg ) ) {

            for ( final int i : abhaengigkeiten.get( arg ) ) {

                update( i );
            }
        }

    }

    /**
     * Wird aufgerufen wenn eine der CheckBoxen angeklickt wird.
     * Aktiviert und deaktiver die Talente eines Helden.
     */
    private class ActionDeAktivierenErschaffung implements ActionListener {

        public void actionPerformed( final ActionEvent e ) {

            final JCheckBox checkBox = (JCheckBox) e.getSource();

            final int index = Integer.valueOf( checkBox.getName() );

            final Talent talent = talente.get( index );

            if ( checkBox.isSelected() ) {
                // Der Benutzer will das Talent aktivieren.

                if ( held.hat( talent ) ) {
                    // Sonderfall Spezial-Talent mit negativer vorgegebener Stufe:
                    // Der Benutzer hat gerade auf die Checkbox
                    // geklickt um ein Talent zu aktivieren. Wie die Abfrage von
                    // held.hat( talent ) aber just ergeben hat besitzt der Held
                    // das Talent schon. Es muss sich dabei also um ein
                    // Spezial-Talent mit negativer Stufe handeln.
                    // Das Talent ist nur noch als aktivert zu markieren und auf
                    // Stufe 0 zu setzen.
                    final TalentWert wert = held.getWert( talent );

                    wert.setAktiviert( true );

                    wert.setStufe( 0 );

                    return;

                }

                // Der normale Fall. Der Held erhaelt das Talent auf Stufe 0 oder
                // der vorgegebenen Mindeststufe neu.
                final int stufe;
                if ( held.hatVorgegeben( talent ) ) {

                    stufe = held.getMindeststufe( talent );

                } else {

                    stufe = 0;

                }

                held.einfuegen( new TalentWert( talent, stufe ) );

            } else {
                // Der Benutzer will das Talent deaktivieren.

                if ( held.hatVorgegeben( talent )
                        && held.getMindeststufe( talent ) < 0
                        && ( ! talent.istBasisTalent() ) ) {
                    // Sonderfall Spezial-Talent mit negativer vorgegebener Stufe:
                    // Das Talent wird nicht entfernt, sonder nur als deaktivert
                    // markiert. Die Stufe wird auf den vorgegebenen negativen Wert
                    // gesetzt oder bleibt auf 0 wenn sie gerade 0 ist.
                    final TalentWert wert = held.getWert( talent );

                    wert.setAktiviert( false );

                    int stufe = wert.getStufe( held );

                    if ( 0 < stufe ) {
                        // Aktuell ist die Stufe positiv, da das Talent deaktiviert
                        // werden soll wird es auf die negative vorgegebene Stufe
                        // gestellt.
                        stufe = held.getMindeststufe( talent );

                    }

                    wert.setStufe( stufe );

                } else {
                    // Der normale Fall. Das Talent wird einfach entfernt.
                    held.entfernen( talent );

                }

            }
        }

    }


    /**
     * Wird aufgerufen wenn eine der CheckBoxen angeklickt wird.
     * Aktiviert und deaktiver die Talente eines Helden.
     */
    private class ActionDeAktivierenSteigern implements ActionListener {

        public void actionPerformed( final ActionEvent e ) {

            final JCheckBox checkBox = (JCheckBox) e.getSource();

            final int index = Integer.valueOf( checkBox.getName() );

            final Talent talent = talente.get( index );

            if ( checkBox.isSelected() ) {

                held.aktivieren( talent, lernmethoden.getLernmethode() );

            } else {

                held.deaktivieren( talent );
            }
        }
    }

    /**
     * Wird aufgerufen wenn der Wert in einem der Spinner veraendert wird.
     */
    private class ActionStufeErschaffung implements ChangeListener {

        public void stateChanged( final ChangeEvent e ) {

            final JSpinner spinner = (JSpinner) e.getSource();

            final int i = Integer.valueOf( spinner.getName() );

            final Talent talent = talente.get( i );

            if ( held.hat( talent ) ) {

                final int stufe = (Integer) spinner.getValue();

                final TalentWert wert = held.getWert( talent );

                wert.setStufe( stufe );
            }
        }
    }

    /**
     * Wird aufgerufen wenn der Wert in einem der Spinner veraendert wird.
     */
    private class ActionStufeSteigern implements ChangeListener {

        public void stateChanged( final ChangeEvent e ) {

            final JSpinner spinner = (JSpinner) e.getSource();

            final int i = Integer.valueOf( spinner.getName() );

            final Talent talent = talente.get( i );

            if ( held.hat( talent ) ) {

                final int stufe = (Integer) spinner.getValue();

                held.setStufe( talent, stufe, lernmethoden.getLernmethode() );
            }
        }
    }
}
