/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.bonus;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelAuswahl< T >
extends PanelBonus
implements Uebersetzbar {

    /**
     * Erklaert dem Benutzer was geweahlt werden soll.
     */
    private final JLabel beschreibung;
    
    /**
     * Der Lokalisierungsschluessel fuer die Beschreibung.
     */
    private String beschreibungL10nSchluessel;
    
    /**
     * Ein Comparator zum sortieren der Elemente aus denen gewaehlt werden soll.
     */
    private Comparator< ? super T > comparator;
    
    /** Die ButtonGroup fuer die Radio Buttons. */
    private final ButtonGroup buttonGroup;

    /** Die RadioButtons mit denen man die Auswahl treffen kann. */
    private final List< JRadioButton > radioButtons;

    /** Die Elemente die zur Auswahl stehen. */
    private final List< T > elemente;

    /** Erzeugt ein leeres Panel. */
    public PanelAuswahl() {

        super();

        setLayout( new BorderLayout() );

        beschreibung = new JLabel();
        
        elemente = new ArrayList< T >();

        radioButtons = new ArrayList< JRadioButton >();

        buttonGroup = new ButtonGroup();
    }

    /**
     * Baut die Benutzeroberflaeche auf.
     *
     * @param beschreibung die dem Benutzer erklaert was zu tun ist.
     */
    public final void aufbauenBenutzeroberflaeche() {

        removeAll();

        initialisiereRadioButtons();

        beschreibung.setText( I18n.i18n( beschreibungL10nSchluessel ) );
        
        final JPanel panel = new JPanel();

        panel.setLayout( erzeugeLayout() );

        final CellConstraints cc = new CellConstraints();

        panel.add( beschreibung, cc.xyw( 2, 2, 3 ) );

        for ( int i = 0, n = elemente.size() ; i < n ; ++i ) {

            final int y = ( 2 * i ) + 4;

            panel.add( radioButtons.get( i ), cc.xy( 3, y ) );
        }

        final JScrollPane scrollPane = Util.erzeugeScrollPane( panel );
        
        add( scrollPane, BorderLayout.CENTER );
    }

    /** Erzeugt ein Layout zur Anordnung der Button und Label. */
    private FormLayout erzeugeLayout() {

        final StringBuilder zeilenLayout = new StringBuilder();

        zeilenLayout.append( "7dlu, P," );
        for ( int i = 0, n = elemente.size(); i < n ; ++i ) {
            zeilenLayout.append( " 4dlu, P," );
        }
        zeilenLayout.append( " 7dlu" );

        return new FormLayout( "7dlu, D:G, P, D:G, 7dlu",
                               zeilenLayout.toString() );

    }

    /**
     * @return das vom Benutzer ausgewaehlte Element.
     */
    public final T getGewaehltesElement() {

        for ( int i = 0, n = radioButtons.size(); i < n ; ++i ) {

            final JRadioButton button = radioButtons.get( i );

            if ( button.isSelected() ) {

                return elemente.get( i );
            }
        }

        // Das sollte eigentlich nicht passieren.
        throw new IllegalStateException(
                "Keiner der RadioButtons ist selektiert." );
    }

    /** @return eine Referenz auf dieses Panel. */
    public final JPanel getPanel() {
        
        return this;
    }
    
    /**
     * Sorgt dafuer das genuegend Radio Buttons vorhanden sind,
     * sie den richtigen Text haben und der erste Button selektiert ist.
     */
    private void initialisiereRadioButtons() {

        if ( radioButtons.size() < elemente.size() ) {

            // Es gibt zu wenige Buttons es muessen mehr erzeugt werden.

            for ( int i = radioButtons.size(), n = elemente.size()
                    ; i < n
                    ; ++i ) {

                final JRadioButton button = new JRadioButton();

                buttonGroup.add( button );

                radioButtons.add( button );
            }
        }

        // Die Button mit Text versehen.
        for ( int i = 0, n = elemente.size() ; i < n ; ++i ) {

            final JRadioButton button = radioButtons.get( i );

            final T element = elemente.get( i );

            button.setText( element.toString() );
        }

        // Den ersten Button auswaehlen.
        radioButtons.get( 0 ).setSelected( true );
    }

    public void neuUebersetzen() {
        
        if ( null != beschreibungL10nSchluessel ) {
            
            beschreibung.setText( I18n.i18n( beschreibungL10nSchluessel ) );
        }
        
        T gewaehltesElement = getGewaehltesElement();
        
        Collections.sort( elemente, comparator );
        
        for ( int i = 0, n = elemente.size() ; i < n ; ++i ) {
            
            JRadioButton radioButton = radioButtons.get( i );
            
            T aktuellesElement = elemente.get( i );
            
            radioButton.setText( aktuellesElement.toString() );
            
            radioButton.setSelected( 
                    gewaehltesElement.equals( aktuellesElement ) );
        }
    }
    
    /**
     * Bestimmt die Elemente die zur Auswahl stehen.
     *
     * @param beschreibungL10nSchluessel Der Schluessel fuer den Text aus der
     *      Lokalisierungsdatei der dem Benutzer angezeigt wird. Er 
     *      kann erklaeren warum oder was gewaehlt werden soll.
     * @param dieElemente die zur Auswahl stehen. Sie werden mittels
     *      {@link Object#toString()} in Text umgewandelt um sie in der
     *      Benutzeroberflaeche anzuzeigen.
     * @param derComparator zum sortieren der uebergebenen Elemente.
     */
    public final void setElemente(
            final String beschreibungL10nSchluessel,
            final List< T > dieElemente,
            final Comparator< ? super T > derComparator ) {

        this.beschreibungL10nSchluessel = beschreibungL10nSchluessel;
        
        this.comparator = derComparator;
        
        elemente.clear();

        elemente.addAll( dieElemente );

        Collections.sort( elemente, comparator );
        
        aufbauenBenutzeroberflaeche();

        validate();
        repaint();
    }
}
