/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.sonderfertigkeiten;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.varianten.VarianteListener;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.util.Beobachter;

public class SonderfertigkeitWerte implements Cloneable {

    /** Der Held dem die Sonderfertigkeiten gehoeren. Kann {@code null} sein */
    private final Held held;

    /**
     * Die gespeicherten Sonderfertigkeiten.
     * Alle Sonderfertigkeiten, die Varianten der selben Sonderfertigkeit
     * darstellen, sind zusammen in einer Liste gespeichert.
     * Die Reihenfolge in der Liste entspricht dabei der Reihenfolge in
     * welcher der Held die Sonderfertigkeiten erhalten hat. Die als erstes
     * erhaltene Sonderfertigkeit steht an Position 0.
     * So kann, zur Berechnung der Kosten einer Talentspezialisierung,
     * festgestellt werden die wievielte Spezialisierung sie ist.
     */
    public 
    Map< Sonderfertigkeit, List< SonderfertigkeitWert > > sonderfertigkeiten;

    /** Erzeugt eine neue Liste fuer Sonderfertigkeiten. */
    public SonderfertigkeitWerte() {
        this ( null );
    }

    /**
     * @param derHeld der Held dem die Sonderfertigkeiten gehoeren. Bei jeder
     *      Veraenderung der Sonderfertigkeiten wird die Methode
     *      {@link Held#notifyObservers()} aufgerufen. Ausnahme ist die
     *      Methode {@link #einfuegen(SonderfertigkeitWert)}.
     *      Das Argument darf {@code null} sein.
     */
    public SonderfertigkeitWerte( final Held derHeld ) {

        held = derHeld;

        sonderfertigkeiten = 
            new HashMap< Sonderfertigkeit, List< SonderfertigkeitWert > >();
    }

    /**
     * Erzeugt eine Deep Copy der Werte.
     */
    public SonderfertigkeitWerte clone() {
        
        SonderfertigkeitWerte clone = null;
        
        try {
            
            clone = (SonderfertigkeitWerte) super.clone();
            
            sonderfertigkeiten = cloneSonderfertigkeiten();
            
        } catch ( CloneNotSupportedException e ) {
            
            DialogFehlermeldung.zeigeDialog( e );
        }
        
        return clone;
    }
    
    /**
     * @return Eine Deep Copy der Map mit den Sonderfertigkeiten.
     */
    public Map< Sonderfertigkeit, List< SonderfertigkeitWert > >
    cloneSonderfertigkeiten() {
        
        final Map< Sonderfertigkeit, List< SonderfertigkeitWert > > clone
        = new HashMap< Sonderfertigkeit, List<SonderfertigkeitWert> >();
        
        for ( final Sonderfertigkeit sonderfertigkeit : 
            sonderfertigkeiten.keySet() ) {
            
            final List< SonderfertigkeitWert > werte =
                sonderfertigkeiten.get( sonderfertigkeit );
                
            clone.put( 
                    sonderfertigkeit, 
                    SonderfertigkeitWert.deepCopy( werte ) );
        }
            
        return clone;
    }
    
    /**
     * Speichert einen weiteren Wert in der Liste.
     *
     * @param wert der gespeichert werden soll. Wenn der Wert schon gespeichert
     *      ist macht die Methode nichts.
     * @return {@code true} wenn der Wert in die Liste eingefuegt wurde, sonst
     *      {@code false}. In letzeterm Fall ist der Wert schon in der Liste
     *      vorhanden.
     * @throws IllegalArgumentException wenn {@code null} uebergeben wird.
     */
    public final boolean einfuegen( final SonderfertigkeitWert wert ) {

        if ( null == wert ) {
            throw new IllegalArgumentException(
                    "null ist als Argument nicht erlaubt." );
        }

        if ( ! enthaelt( wert ) ) {
            // Nur wenn der Wert noch nicht gespeichert ist.

            final Sonderfertigkeit sonderfertigkeit;
            sonderfertigkeit = wert.getSonderfertigkeit();

            final SonderfertigkeitWert neuerWert = wert.clone();

            if ( null != held ) {
                
                neuerWert.anmelden( (Beobachter) held );
                neuerWert.anmelden( (VarianteListener) held );
            }

            if ( ! enthaelt( sonderfertigkeit ) ) {

                sonderfertigkeiten.put(
                        sonderfertigkeit,
                        new ArrayList< SonderfertigkeitWert>( 1 ) );
            }

            sonderfertigkeiten.get( sonderfertigkeit ).add( neuerWert );

            /*
             * Hier kein notifyObservers aufrufen. Der Aufruf wurde in die
             * Klasse Held verlegt, weil hier, zu diesem Zeitpunkt die Listen
             * ortskenntnisse und gelaendekunden noch nicht aktualisiert sind.
             * Die Observer wuerden somit nicht die aktuellen Informationen
             * erhalten
            if ( null != held ) {

                held.notifyObservers();
            }
            */
            return true;
        }

        return false;
    }

    /**
     * Entfernt eine Sonderfertigkeit aus der Liste. Im folgenden sind alle
     * Varianten der Sonderfertigkeit aus der Liste entfernt.
     *
     * @param sonderfertigkeit deren Varianten entfernt werden sollen.
     */
    public final void entfernen( final Sonderfertigkeit sonderfertigkeit ) {

        if ( enthaelt( sonderfertigkeit ) ) {

            for ( final SonderfertigkeitWert wert :
                sonderfertigkeiten.remove( sonderfertigkeit ) ) {

                if ( null != held ) {
                    
                    wert.abmelden( (Beobachter) held );
                    wert.abmelden( (VarianteListener) held );
                }
            }

            if ( null != held ) {

                held.notifyObservers( sonderfertigkeit );
            }
        }
    }

    /**
     * Entfernt eine Sonderfertigkeit aus der Liste.
     * Wenn die Sonderfertigkeit nicht in der Liste ist, macht die Methode
     * nichts.
     *
     * @param wert der entfernt werden soll.
     */
    public final void entfernen( final SonderfertigkeitWert wert ) {

        if ( enthaelt( wert ) ) {

            final Sonderfertigkeit sonderfertigkeit;
            sonderfertigkeit = wert.getSonderfertigkeit();

            final List< SonderfertigkeitWert > werte;
            werte = sonderfertigkeiten.get( sonderfertigkeit );

            for ( int i = 0, n = werte.size() ; i < n ; ++i ) {

                if ( werte.get( i ).hatGleicheSfUndVariante( wert ) ) {

                    SonderfertigkeitWert entfernterWert = werte.remove( i );
                    
                    if ( null != held ) {
                        
                        entfernterWert.abmelden( (Beobachter) held );
                        entfernterWert.abmelden( (VarianteListener) held );
                    }
                    
                    break;
                }

            }

            if ( 0 == werte.size() ) {

                sonderfertigkeiten.remove( sonderfertigkeit );

            }

            if ( null != held ) {

                held.notifyObservers( sonderfertigkeit );
            }
        }
    }

    /**
     * Prueft ob eine Sonderfertigkeit gespeichert ist.
     *
     * @param sonderfertigkeit fuer die geprueft werden soll ob sie gespeichert
     *      ist.
     * @return {@code true} wenn die Sonderfertigkeit gespeichert ist, sonst
     *      {@code false}.
     */
    public final boolean enthaelt( final Sonderfertigkeit sonderfertigkeit ) {
        return sonderfertigkeiten.containsKey( sonderfertigkeit );
    }

    /**
     * Prueft ob der Wert gespeichert ist.
     *
     * @param wert fuer den geprueft werden soll ob er gespeichert ist.
     * @return {@code true} wenn der Wert gespeichert ist, sonst {@code false}.
     */
    public final boolean enthaelt( final SonderfertigkeitWert wert ) {

        final Sonderfertigkeit sonderfertigkeit = wert.getSonderfertigkeit();

        if ( ! enthaelt( sonderfertigkeit ) ) {
            // Die Sonderfertigkeit ist hier nicht enthalten.
            // Es kann abgebrochen werden.
            return false;
        }

        for ( final SonderfertigkeitWert w :  getWerte( sonderfertigkeit ) ){
            
            if ( w.hatGleicheSfUndVariante( wert ) ) {
                
                return true;
            }
        }
        
        return false;
    }
    
    @Override
    public boolean equals( Object obj ) {
        
        if ( null == obj 
                || ! obj.getClass().equals( getClass() ) ) {
            
            return false;
        }
        
        final SonderfertigkeitWerte werte = (SonderfertigkeitWerte) obj;
        
        return sonderfertigkeiten.equals( werte.sonderfertigkeiten );
    }

    /**
     * Liefert fuer einen Wert als wievielte Variante, der Sonderfertigkeit
     * des Wertes, er hier gespeichert wurde.
     * Ist der Wert hier nicht gespeichert, dann liefert die Methode, die
     * wievielte Variante der Wert waere, wenn er als naechstes hier
     * gespeichert wuerde.
     *
     * @param wert dessen Index ermittelt werden soll.
     * @return den Index des Wertes. Es wird bei 1 angefangen zu zaehlen.
     */
    public final int getIndex( final SonderfertigkeitWert wert ) {

        final Sonderfertigkeit sonderfertigkeit = wert.getSonderfertigkeit();

        if ( ! enthaelt( sonderfertigkeit ) ) {
            
            return 1;
        }

        final List< SonderfertigkeitWert > werte = getWerte( sonderfertigkeit );
        
        for ( int i = 0, n = werte.size() ; i < n ; ++i ) {
            
            if ( werte.get( i ).hatGleicheSfUndVariante( wert ) ) {

                return 1 + i;

            }
        }

        return werte.size();
    }

    /**
     * @param wert der gesucht ist.
     * @return den in der Liste gespeicherten Wert oder {@code null} wenn der
     *      Wert nicht in der Liste gespeichert ist. Es handelt sich dabei
     *      wirklich um das Objekt das in der Liste gespeichert ist.
     *      Aenderungen an diesem Objekt andern den Wert in der Liste.
     */
    public final SonderfertigkeitWert getWert(
            final SonderfertigkeitWert wert ) {

        if ( enthaelt( wert ) ) {

            final Sonderfertigkeit sonderfertigkeit;
            sonderfertigkeit = wert.getSonderfertigkeit();

            for ( final SonderfertigkeitWert w : getWerte( sonderfertigkeit )){

                if ( w.hatGleicheSfUndVariante( wert ) ) {

                    return w;
                }
            }
        }

        return null;
    }

    /**
     * @return alle gespeicherten Werte.
     */
    public final List< SonderfertigkeitWert > getWerte() {

        final List< SonderfertigkeitWert > werte;
        werte = new ArrayList< SonderfertigkeitWert >();

        for ( final List< SonderfertigkeitWert > liste :
            sonderfertigkeiten.values() ) {

            werte.addAll( liste );
        }

        return werte;
    }

    /**
     * @param sonderfertigkeit deren Werte gesucht sind.
     * @return die zur Sonderfertigkeit gehoerenden Werte, oder eine leere
     *      Liste falls die Sonderfertigkeit nicht gespeichert ist.
     */
    private List< SonderfertigkeitWert > getWerte(
            final Sonderfertigkeit sonderfertigkeit ) {

        if ( enthaelt( sonderfertigkeit ) ) {

            return new ArrayList< SonderfertigkeitWert > (
                    sonderfertigkeiten.get( sonderfertigkeit ) );

        } else {

            return new ArrayList< SonderfertigkeitWert >();

        }
    }
}
