/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.talente.voraussetzungen;

import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

public abstract class Voraussetzung {

    /**
     * Verlangt dass der Held die Sprache Garethi oder Tulamidya auf einer
     * Stufe von >= 6 besitzt, wenn er das Talent, welches die Voraussetzung
     * besitzt, auf eine Stufe > 10 steigern will.
     */
    public static final VoraussetzungSteigern GARETHI_TULAMIDYA_6_STEIGERN =
        new VoraussetzungSteigern( 6, Talent.GARETHI, Talent.TULAMIDYA );

    /**
     * Verlangt, dass der Held den Vorteil Gefahreninstinkt besitzt.
     */
    public static final Voraussetzung GEFAHRENINSTINKT =
        new VoraussetzungVorteil( VorNachteil.GEFAHRENINSTINKT );

    /**
     * Verlangt dass der Held das Talent Gesteinskunde auf eine Stufe von
     * >= 4 besitzt.
     */
    public static final Voraussetzung GESTEINSKUNDE_4 =
        new VoraussetzungAktivierung( Talent.GESTEINSKUNDE );

    /**
     * Verlangt dass der Held das Talent Gesteinskund auf einer Stufe von
     * >= 4 besitzt, wenn er das Talent, welches die Voraussetzung besitzt,
     * auf eine Stufe > 10 steigern will.
     */
    public static final Voraussetzung GESTEINSKUNDE_4_STEIGERN =
        new VoraussetzungSteigern( 4, Talent.GESTEINSKUNDE );

    /**
     * Verlangt dass der Held das Talent Holzbearbeitung auf eine Stufe von
     * >= 4 besitzt.
     */
    public static final Voraussetzung HOLZBEARBEITUNG_4 =
        new VoraussetzungAktivierung( Talent.HOLZBEARBEITUNG );

    /**
     * Verlangt dass der Held das Talent Holzbearbeitung auf einer Stufe von
     * >= 4 besitzt, wenn er das Talent, welches die Voraussetzung besitzt,
     * auf eine Stufe > 10 steigern will.
     */
    public static final Voraussetzung HOLZBEARBEITUNG_4_STEIGERN =
        new VoraussetzungSteigern( 4, Talent.HOLZBEARBEITUNG );

    /**
     * Prueft bei einer Nahkampftechnik ob die Punkte fuer Attacke und Parade
     * korrekt verteilt wurden.
     */
    public static final Voraussetzung KAMPFTECHNIK =
        new VoraussetzungKampftechnik();

    /**
     * Verlangt dass der Held das Talent Koerperbeherrschung auf eine Stufe von
     * >= 4 besitzt.
     */
    public static final Voraussetzung KOERPERBEHERSCHUNG_4 =
        new VoraussetzungAktivierung( Talent.KOERPERBEHERRSCHUNG );

    /**
     * Verlangt dass der Held eine passende Kultursprache auf einer Stufe von
     * >= 6 besitzt, wenn er das Talent, welches die Voraussetzung besitzt,
     * auf eine Stufe > 10 steigern will.
     */
    public static final Voraussetzung KULTURSPRACHE_6_STEIGERN = new VoraussetzungSteigern(
                6,
                Talent.KUSLIKER_ZEICHEN,
                Talent.TULAMIDYA_SCHRIFT,
                Talent.UR_TULAMIDYA_SCHRIFT );

    /**
     * Verlangt, dass der Held den Vorteil Magiegespuer besitzt.
     */
    public static final Voraussetzung MAGIEGESPUER =
        new VoraussetzungVorteil( VorNachteil.MAGIEGESPUER );

    /**
     * Verlangt dass der Held das Talent Malen/Zeichnen auf eine Stufe von
     * >= 4 besitzt.
     */
    public static final Voraussetzung MALEN_ZEICHNEN_4 =
        new VoraussetzungAktivierung( Talent.MALEN_ZEICHNEN );

    /**
     * Verlangt dass der Held das Talent Malen/Zeichnen auf einer Stufe von
     * >= 6 besitzt, wenn er das Talent, welches die Voraussetzung besitzt,
     * auf eine Stufe > 10 steigern will.
     */
    public static final Voraussetzung MALEN_ZEICHNEN_6_STEIGERN =
        new VoraussetzungSteigern( 6, Talent.MALEN_ZEICHNEN );

    /**
     * Verlangt dass der Held das Talent Menschenkenntnis auf eine Stufe von
     * >= 4 besitzt.
     */
    public static final Voraussetzung MENSCHENKENNTNIS_4 =
        new VoraussetzungAktivierung( Talent.MENSCHENKENNTNIS );

    /**
     * Verlangt dass der Held das Talent Menschenkenntnis auf eine Stufe von
     * >= 4 besitzt, wenn er das Talent, welches die Voraussetzung besitzt,
     * auf eine Stufe > 10 steigern will.
     */
    public static final Voraussetzung MENSCHENKENNTNIS_4_STEIGERN =
        new VoraussetzungSteigern( 4, Talent.MENSCHENKENNTNIS );

    /** Fuer Talente die nicht mit Einarmig kombiniert werden koennen. */
    public static final Voraussetzung NICHT_EINARMIG =
        new VoraussetzungNichtVorNachteil( VorNachteil.EINARMIG );

    /** Fuer Talente die nicht frei gewaehlt werden koennen. */
    public static final Voraussetzung NICHT_FREI_WAEHLBAR =
        new VoraussetzungNichtFreiWaehlbar();

    /** Fuer Talente die nicht mit Totenangst kombiniert werden koennen. */
    public static final Voraussetzung NICHT_TOTENANGST =
        new VoraussetzungNichtVorNachteil( VorNachteil.TOTENANGST );

    /**
     * Verlangt, dass der Held die Gabe Prophezeien besitzt.
     */
    public static final Voraussetzung PROPHEZEIEN =
        new VoraussetzungVorteil( VorNachteil.PROPHEZEIEN );

    /**
     * Verlangt dass der Held das Talent Rechnen auf eine Stufe von >= 4
     * besitzt.
     */
    public static final Voraussetzung RECHNEN_4 =
        new VoraussetzungAktivierung( Talent.RECHNEN );

    /**
     * Verlangt dass der Held das Talent Rechnen auf einer Stufe von
     * >= 6 besitzt, wenn er das Talent, welches die Voraussetzung besitzt,
     * auf eine Stufe > 10 steigern will.
     */
    public static final Voraussetzung RECHNEN_6_STEIGERN =
        new VoraussetzungSteigern( 6, Talent.RECHNEN );

    /**
     * Verlangt dass der Held eine Schrift auf eine Stufe von >= 4 besitzt.
     */
    public static final Voraussetzung SCHRIFT_4 =
        new VoraussetzungAktivierungSchrift();

    /**
     * Verlangt dass der Held eine Schrift auf einer Stufe von >= 6 besitzt,
     * wenn er das Talent, welches die Voraussetzung besitzt, auf eine Stufe
     * > 10 steigern will.
     */
    public static final Voraussetzung SCHRIFT_6_STEIGERN =
        new VoraussetzungSteigernSchrift();

    /**
     * Verlangt dass der Held das Talent Sinnenschaerfe auf eine Stufe von
     * >= 4 besitzt.
     */
    public static final Voraussetzung SINNENSCHAERFE_4 =
        new VoraussetzungAktivierung( Talent.SINNENSCHAERFE );

    /**
     * Verlangt dass der Held das Talent Sinnenschaerfe auf einer Stufe von
     * >= 4 besitzt, wenn er das Talent, welches die Voraussetzung besitzt,
     * auf eine Stufe > 10 steigern will.
     */
    public static final Voraussetzung SINNENSCHAERFE_4_STEIGERN =
        new VoraussetzungSteigern( 4, Talent.SINNENSCHAERFE );

    /**
     * Verlangt dass der Held das Talent Sinnenschaerfe auf einer Stufe von
     * >= 6 besitzt, wenn er das Talent, welches die Voraussetzung besitzt,
     * auf eine Stufe > 10 steigern will.
     */
    public static final Voraussetzung SINNENSCHAERFE_6_STEIGERN =
        new VoraussetzungSteigern( 6, Talent.SINNENSCHAERFE );

    /**
     * Die Standard-Voraussetzung fuer alle Talente. Sie prueft ob der Held
     * ein Talent das er besitzen muss auch besitzt und ob die Stufe des
     * Talents in den vorgegebenen Grenzen liegt.
     */
   public static final Voraussetzung STANDARD = new VoraussetzungStandard();

    /**
     * Verlangt dass der Held das Talent Tierkunde auf einer Stufe von
     * >= 4 besitzt, wenn er das Talent, welches die Voraussetzung besitzt,
     * auf eine Stufe > 10 steigern will.
     */
    public static final Voraussetzung TIERKUNDE_4_STEIGERN =
        new VoraussetzungSteigern( 4, Talent.TIERKUNDE );

    /**
     * Verlangt dass der Held das Talent Wildnisleben auf einer Stufe von
     * >= 4 besitzt, wenn er das Talent, welches die Voraussetzung besitzt,
     * auf eine Stufe > 10 steigern will.
     */
    public static final Voraussetzung WILDNISLEBEN_4_STEIGERN =
        new VoraussetzungSteigern( 4, Talent.WILDNISLEBEN );

    /**
     * Verlangt, dass der Held den Vorteil Zwergennase besitzt.
     */
    public static final Voraussetzung ZWERGENNASE =
        new VoraussetzungVorteil( VorNachteil.ZWERGENNASE );

    static {
        GARETHI_TULAMIDYA_6_STEIGERN.ergaenzeAbhaengigkeit( Eigenschaft.KL );
    }

    /**
     * @return die Regelelemente von denen die Voraussetzung abhaengig ist.
     */
    public abstract Collection< Object > getAbhaengigkeiten();

    /**
     * Prueft ob der Held die Voraussetzung des Talents erfuellt.
     *
     * @param held der die Voraussetzung erfuellen muss.
     * @param talent das die Voraussetzung hat.
     * @return Eine Beschreibung der Voraussetzung wenn sie nicht erfuellt
     *      wird.
     */
    public abstract KonfliktBeschreibung pruefen(
            final Held held,
            final Talent talent );
}
