/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.kosten;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.varianten.VariantenTalentgruppe;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

import java.util.Collection;
import java.util.Collections;

class KostenUnfaehigkeitTalentgruppe implements Kosten {

    /**
     * Die Kosten fuer Nahkampffertigkeiten und alle Talentgruppen ausser Kampf
     * und Koerper.
     */
    static final int GP_KOSTEN_ANDERE_NAHKAMPF = -10;

    /** Die Kosten fuer Fernkampftalente. */
    static final int GP_KOSTEN_FERNKAMPF = -5;

    /** Die Kosten fuer die Talentgruppen Kampf und Koerper. */
    static final int GP_KOSTEN_KAMPF_KOERPER = -15;

    /**
     * Die Kosten fuer die Talentgruppen Sprachen und Schriften.
     */
    static final int GP_KOSTEN_SPRACHEN_SCHRIFTEN = -7;

    /**
     * Wenn der Held eine vorgegebene Unfaehigkeit fuer ein Talent der
     * {@code talentgruppe} besitzt, so werden die Kosten der Unfaehigkeit
     * angerechnet.
     *
     * @param held
     *            der die Unfaehigkeit fuer [Talentgruppe] erwerben will.
     * @param talentgruppe
     *            in der ein eine Unfaehigkeit fuer [Talent] gesucht ist.
     * @return die Kosten die dem Helden aufgrund eine Unfaehigkeit fuer
     *         [Talent] angerechnet werden oder 0 wenn es nicht anzurechnen
     *         gibt.
     */
    private int angerechneteKosten(
            final Held held,
            final Object talentgruppe ) {

        for ( final VorNachteilWert unfaehigkeitTalent :
                            held.getWerte( VorNachteil.UNFAEHIGKEIT_TALENT ) ) {

            final Talent talent = 
                (Talent) unfaehigkeitTalent.getVariante().getElement();

            if ( held.hatVorgegeben( unfaehigkeitTalent )
                    && istInTalentgruppe( talent, talentgruppe ) ) {

                return unfaehigkeitTalent.getGpKosten();
            }
        }

        return 0;
    }

    /** {@inheritDoc} */
    public int getAbbaukosten(
            final Held held,
            final VorNachteilWert wert,
            final Lernmethode lernmethode ) {
        if ( lernmethode == Lernmethode.FREI )
            return 0;

        return -100 * getGpKosten( wert );
    }

    /** @return eine leere Liste. */
    @SuppressWarnings( "unchecked" )
    public Collection<Object> getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /** {@inheritDoc} */
    public int getGpKosten() {
        return GP_KOSTEN_FERNKAMPF;
    }

    /** {@inheritDoc} */
    public final int getGpKosten( final VorNachteilWert wert ) {

        final Object talentgruppe = wert.getVariante();

        if ( talentgruppe.equals( Talentgruppe.KAMPF )
          || talentgruppe.equals( Talentgruppe.KOERPER ) ) {

            return GP_KOSTEN_KAMPF_KOERPER;

        } else if ( talentgruppe.equals( VariantenTalentgruppe.FERNKAMPF ) ) {

            return GP_KOSTEN_FERNKAMPF;

        } else if ( talentgruppe.equals( Talentgruppe.SCHRIFTEN )
                || talentgruppe.equals( Talentgruppe.SPRACHEN ) ) {

            return GP_KOSTEN_SPRACHEN_SCHRIFTEN;

        } else {

            return GP_KOSTEN_ANDERE_NAHKAMPF;
        }
    }

    /** {@inheritDoc} */
    public final int getGpKosten(
            final Held held,
            final VorNachteilWert wert ) {

        if ( held.hatVorgegeben( wert ) ) {

            return 0;
        }

        final int angerechnet =
            angerechneteKosten( held, wert.getVariante() );

        return getGpKosten( wert ) - angerechnet;
    }

    /** @return 0 */
    public int getSenkungskosten(final Lernmethode lernmethode, final VorNachteilWert wert ) {
        return 0;
    }

    /**
     * Prueft ob das uebergebene Talent in der Uebergebenen Talentgruppe ist.
     *
     * @param talent
     *            das ueberprueft werden soll.
     * @param talentgruppe
     *            die ueberprueft werden soll.
     * @return {@code true}, wenn das Talent zu der Talentgruppe gehoert, sonst
     *         {@code false}.
     */
    private boolean istInTalentgruppe(
            final Talent talent,
            final Object talentgruppe ) {

        final Talentgruppe talentgruppeTalent = talent.getTalentgruppe();

        return talentgruppeTalent.equals( talentgruppe )
            || istNahkampfTechnikUndTgNahkampf( talent, talentgruppe )
            || istFernkampfTechnikUndTgFernkampf( talent, talentgruppe );

    }

    /**
     * Prueft ob es sich bei eine Talent um eine Fernkampf-Technik und bei
     * einer Talentgruppe um die Talentgruppe Fernkampffertigkeiten handelt.
     *
     * @param talent
     *            das ueberprueft wird ob es eine Fernkampf-Technik ist.
     * @param talentgruppe
     *            die ueberprueft wird ob sie
     *            {@link VariantenTalentgruppe#FERNKAMPF} ist.
     * @return {@code true} wenn das Talent eine Fernkampf-Technik ist und die
     *         Talentgruppe {@link VariantenTalentgruppe#FERNKAMPF}.
     */
    private boolean istFernkampfTechnikUndTgFernkampf(
            final Talent talent,
            final Object talentgruppe ) {

        return Talentgruppe.KAMPF.equals( talent.getTalentgruppe() )
            && ((Kampftechnik) talent).istFernkampfTechnik()
            && talentgruppe.equals( VariantenTalentgruppe.FERNKAMPF );
    }

    /**
     * Prueft ob es sich bei eine Talent um eine Nahkampf-Technik und bei
     * einer Talentgruppe um die Talentgruppe Nahkampffertigkeiten handelt.
     *
     * @param talent
     *            das ueberprueft wird ob es eine Nahkampf-Technik ist.
     * @param talentgruppe
     *            die ueberprueft wird ob sie
     *            {@link VariantenTalentgruppe#NAHKAMPF} ist.
     * @return {@code true} wenn das Talent eine Nahkampf-Technik ist und die
     *         Talentgruppe {@link VariantenTalentgruppe#NAHKAMPF}.
     */
    private boolean istNahkampfTechnikUndTgNahkampf(
            final Talent talent,
            final Object talentgruppe ) {

        return Talentgruppe.KAMPF.equals( talent.getTalentgruppe() )
            && ((Kampftechnik) talent).istNahkampfTechnik()
            && talentgruppe.equals( VariantenTalentgruppe.NAHKAMPF );
    }

    /** @return {@code false} */
    public final boolean istSenkbar( final VorNachteilWert wert ) {
        return false;
    }
}
