/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.voraussetzungen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.basis.varianten.VariantenTalentgruppe;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class VoraussetzungBegabungUnfaehigkeitTalent extends Voraussetzung {

    /** Begabung oder Unfaehigkeit fuer [Talent]. */
    private final VorNachteil inkompatiblerVorNachteil;
    
    /**
     * @param derInkompatibleVorNachteil
     *            Soll die Voraussetzung fuer Begabung fuer [Talent] sein, so
     *            muss hier Unfaehigkeit fuer [Talent] uebergeben werden. Soll
     *            die Voraussetzung fuer Unfaehigkeit fuer [Talent] sein, so
     *            muss hier Begabung fuer [Talent] uebergeben werden.
     */
    VoraussetzungBegabungUnfaehigkeitTalent(
            final VorNachteil derInkompatibleVorNachteil ) {
        
        if ( ( derInkompatibleVorNachteil != VorNachteil.BEGABUNG_TALENT ) 
          && ( derInkompatibleVorNachteil != VorNachteil.UNFAEHIGKEIT_TALENT )){
            
            throw new IllegalArgumentException( String.format(
                    "Es durfen nur Begbung/Unfaehigkeit fuer [Talent] " +
                    "uebergeben werden. Es wurde aber %1$s uebergeben", 
                    derInkompatibleVorNachteil.getName() ) );
        }
        
        inkompatiblerVorNachteil = derInkompatibleVorNachteil;
    }

    /** 
     * @return Begabung Talent/Talentgruppe und Unfaehigkeit 
     *         Talent/Talentgruppe.
     */
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList( 
                (Object) VorNachteil.BEGABUNG_TALENT,
                VorNachteil.BEGABUNG_TALENTGRUPPE,
                VorNachteil.UNFAEHIGKEIT_TALENT,
                VorNachteil.UNFAEHIGKEIT_TALENTGRUPPE  );
    }

    /**
     * Prueft ob {@code wert} fuer den Helden eine vorgegebene Unfaehigkeit fuer
     * [Talent] ist.
     * 
     * @param held
     *            der ueberprueft wird.
     * @param wert
     *            der darauf geprueft wird ob er fuer den Helden eine
     *            vorgegebene Unfaehigkeit fuer [Talent] ist.
     * @return {@code true}, wenn der Wert eine vorgegeben Unfaehigkeit fuer
     *         [Talent] ist, sonst {@code false}.
     */
    private boolean istVorgegebeneUnfaehigkeitTalent(
            final Held held, 
            final VorNachteilWert wert) {
        
        return held.hatVorgegeben( wert ) 
          && wert.getVorNachteil().equals( VorNachteil.UNFAEHIGKEIT_TALENT );
    }
    
    /**
     * Prueft ob der Held eine Begabung fuer [Talentgruppe] besitzt, die mit
     * der Begabung/Unfaehigkeit fuer ein Talent inkompatibel ist.
     * 
     * @param held
     *            der ueberprueft werden soll.
     * @param talent
     *            zu dessen Begabung/Unfaehigkeit eine inkompatible Begabung
     *            fuer [Talentgruppe] gesucht wird.
     * @param beschreibung
     *            in die das Problem eingetragen wird, wenn der Held eine
     *            inkompatible Begabung fuer [Talentgruppe] besitzt.
     */
    private void pruefeBegabungTalentgruppe(
            final Held held,
            final Talent talent,
            final KonfliktBeschreibung beschreibung ) {
        
        final Talentgruppe talentgruppe = talent.getTalentgruppe();
        
        final VorNachteilWert wert = new VorNachteilWert(
                VorNachteil.BEGABUNG_TALENTGRUPPE,
                new Variante( talentgruppe ) );
        
        pruefeTalentgruppe( held, talent, wert, beschreibung );
    }

    /**
     * Prueft ob der Held den Vor-/Nachteil {@link #inkompatiblerVorNachteil}
     * in der Variante {@code talent} besitzt.
     * 
     * @param held
     *            der ueberprueft werden soll.
     * @param talent
     *            Die Variante fuer {@link #inkompatiblerVorNachteil}.
     * @param beschreibung
     *            in die eingetragen wird, wenn der Held den inkompatiblen
     *            Vor-/Nachteil besitzt.
     */
    private void pruefeBegabungUnfaehigkeitTalent(
            final Held held,
            final Talent talent,
            final KonfliktBeschreibung beschreibung ) {
        
        final VorNachteilWert wert = new VorNachteilWert( 
                inkompatiblerVorNachteil,
                new Variante( talent ) );
        
        if ( held.hat( wert ) ) {
            
            beschreibung.einfuegen( String.format(
                    VoraussetzungNichtKombinierbar.getNichtKombinierbar(),
                    wert ) );
        }
    }

    /**
     * Prueft ob der Held eine inkompatible Begabung oder Unfaehigkeit besitzt.
     */
    public final KonfliktBeschreibung pruefen(
            final Held held, 
            final VorNachteilWert wert ) {
        
        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();
        
        final Talent talent = (Talent) wert.getVariante().getElement();
        
        pruefeBegabungTalentgruppe( held, talent, beschreibung );

        pruefeBegabungUnfaehigkeitTalent( held, talent, beschreibung );
        
        if ( ! istVorgegebeneUnfaehigkeitTalent( held, wert ) ) {
            // Eine vorgegebene Unfaehigkeit in einem Talent darf zu einer
            // Unfaehigkeit der ganzen Gruppe ausgeweitet werden.
            pruefeUnfaehigkeitTalentgruppe( held, talent, beschreibung );
        }
        
        return beschreibung;
    }
    
    /**
     * Prueft ob der Held eine Begabung/Unfaehigkeit fuer [Talentgruppe]
     * in der Talentgruppe eines Talents besitzt.
     * 
     * @param held
     *            der ueberprueft werden soll.
     * @param talent
     *            zu dessen Talentgruppe die Begabung/Unfaehigkeit gesucht 
     *            wird.
     * @param wert
     *            der Vorteil Begabung fuer [Talentgruppe] oder der Nachteil
     *            Unfaehigkeit fuer [Talentgruppe] fuer die Talentgruppe von
     *            {@link talent}.
     * @param beschreibung
     *            in die das Problem eingetragen wird, wenn der Held eine
     *            inkompatible Begabung/Unfaehigkeit fuer [Talentgruppe] 
     *            besitzt.
     */
    private void pruefeTalentgruppe(
            final Held held,
            final Talent talent,
            final VorNachteilWert wert,
            final KonfliktBeschreibung beschreibung ) {
                
        if ( held.hat( wert ) ) {
            
            beschreibung.einfuegen( String.format(
                    VoraussetzungNichtKombinierbar.getNichtKombinierbar(),
                    wert ) );
    
        } else if ( Talentgruppe.KAMPF.equals( talent.getTalentgruppe() ) ) {
            
            final Kampftechnik kampftechnik = (Kampftechnik) talent;
            
            if ( kampftechnik.istNahkampfTechnik() ) {
                
                wert.setVariante( VariantenTalentgruppe.NAHKAMPF );
                
                if ( held.hat( wert ) ) {
            
                    beschreibung.einfuegen( String.format(
                            VoraussetzungNichtKombinierbar.getNichtKombinierbar(),
                            wert ) );
                }
                
            } else {
                // Es ist eine Fernkampf-Technik.
                
                wert.setVariante( VariantenTalentgruppe.FERNKAMPF );
                
                if ( held.hat( wert ) ) {
                    
                    beschreibung.einfuegen( String.format(
                            VoraussetzungNichtKombinierbar.getNichtKombinierbar(),
                            wert ) );
                }
            }
        }
    }

    /**
     * Prueft ob ein Held eine Unfaehigkeit fuer [Talentgruppe] in der 
     * Talentgruppe von {@code talent} besitzt.
     * 
     * @param held
     *            der ueberprueft werden soll.
     * @param talent
     *            zu dessen Talentgruppe die Unfaehigkeit gesucht wird.
     * @param beschreibung
     *            in die eingetragen wird, wenn der Held eine inkompatible
     *            Unfaehigkeit besitzt.
     */
    private void pruefeUnfaehigkeitTalentgruppe(
            final Held held,
            final Talent talent,
            final KonfliktBeschreibung beschreibung ) {
        
        final Talentgruppe talentgruppe = talent.getTalentgruppe();
        
        final VorNachteilWert wert = new VorNachteilWert(
                VorNachteil.UNFAEHIGKEIT_TALENTGRUPPE,
                new Variante( talentgruppe ) );
        
        pruefeTalentgruppe( held, talent, wert, beschreibung );
    }
}
